from __future__ import annotations

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import (
    extend_schema,
    extend_schema_view,
)
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.permissions import (
    IsAuthenticated,
)
from rest_framework.response import Response

from blog.filters.comment import BlogCommentFilter
from blog.models.comment import BlogComment
from blog.models.post import BlogPost
from blog.serializers.comment import (
    BlogCommentDetailSerializer,
    BlogCommentLikedCommentsRequestSerializer,
    BlogCommentLikedCommentsResponseSerializer,
    BlogCommentMyCommentRequestSerializer,
    BlogCommentSerializer,
    BlogCommentWriteSerializer,
)
from blog.serializers.post import (
    BlogPostDetailSerializer,
)
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods


@extend_schema_view(
    **create_schema_view_config(
        model_class=BlogComment,
        display_config={
            "tag": "Blog Comments",
        },
        serializers={
            "list_serializer": BlogCommentSerializer,
            "detail_serializer": BlogCommentDetailSerializer,
            "write_serializer": BlogCommentWriteSerializer,
        },
        error_serializer=ErrorResponseSerializer,
        additional_responses={
            "create": {201: BlogCommentDetailSerializer},
            "update": {200: BlogCommentDetailSerializer},
            "partial_update": {200: BlogCommentDetailSerializer},
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class BlogCommentViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = BlogComment.objects.all()
    serializers = {
        "default": BlogCommentDetailSerializer,
        "list": BlogCommentSerializer,
        "retrieve": BlogCommentDetailSerializer,
        "create": BlogCommentWriteSerializer,
        "update": BlogCommentWriteSerializer,
        "partial_update": BlogCommentWriteSerializer,
        "replies": BlogCommentSerializer,
        "thread": BlogCommentSerializer,
        "update_likes": BlogCommentDetailSerializer,
        "post": BlogPostDetailSerializer,
        "liked_comments": BlogCommentLikedCommentsResponseSerializer,
        "my_comments": BlogCommentSerializer,
        "my_comment": BlogCommentDetailSerializer,
    }
    response_serializers = {
        "create": BlogCommentDetailSerializer,
        "update": BlogCommentDetailSerializer,
        "partial_update": BlogCommentDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = BlogCommentFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "likes_count",
        "replies_count",
    ]
    ordering = ["-created_at"]
    search_fields = ["translations__content"]

    def get_queryset(self):
        queryset = (
            BlogComment.objects.select_related("user", "post")
            .prefetch_related("likes", "translations")
            .filter(approved=True)
        )
        return queryset

    def get_permissions(self):
        permission_classes = []
        if self.action in [
            "create",
            "update",
            "partial_update",
            "destroy",
            "update_likes",
            "liked_comments",
            "my_comments",
            "my_comment",
        ]:
            permission_classes.append(IsAuthenticated)
        return [permission() for permission in permission_classes]

    @extend_schema(
        operation_id="listBlogCommentReplies",
        summary=_("Get comment replies"),
        description=_(
            "Get all replies (children) of this comment in threaded structure."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def replies(self, request, pk=None):
        comment = self.get_object()
        queryset = (
            comment.get_children()
            .select_related("user", "post")
            .prefetch_related("likes")
            .filter(approved=True)
            .order_by("created_at")
        )
        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="getBlogCommentThread",
        summary=_("Get comment thread"),
        description=_(
            "Get the complete thread (all ancestors and descendants) of this comment."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
        },
    )
    @action(detail=True, methods=["GET"])
    def thread(self, request, pk=None):
        comment = self.get_object()
        ancestors = comment.get_ancestors().select_related("user", "post")
        descendants = comment.get_descendants().select_related("user", "post")

        queryset = [*list(ancestors), comment, *list(descendants)]
        queryset = sorted(queryset, key=lambda x: x.created_at)

        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="toggleBlogCommentLike",
        summary=_("Toggle comment like"),
        description=_("Like or unlike a comment. Toggles the like status."),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentDetailSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["POST"])
    def update_likes(self, request, pk=None):
        comment = self.get_object()
        user = request.user

        if comment.likes.filter(id=user.id).exists():
            comment.likes.remove(user)
            liked = False
        else:
            comment.likes.add(user)
            liked = True

        comment.refresh_from_db()

        serializer = self.get_serializer(comment)
        data = serializer.data
        data["action"] = "liked" if liked else "unliked"

        return Response(data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="getBlogCommentPost",
        summary=_("Get comment's blog post"),
        description=_("Get the blog post that this comment belongs to."),
        tags=["Blog Comments"],
        responses={
            200: BlogPostDetailSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def post(self, request, pk=None):
        comment = self.get_object()
        post = comment.post
        serializer = BlogPostDetailSerializer(
            post, context={"request": request}
        )
        return Response(serializer.data)

    @extend_schema(
        operation_id="checkBlogCommentLikes",
        summary=_("Check bulk like status"),
        description=_(
            "Check which comments from a list are liked by the current user."
        ),
        tags=["Blog Comments"],
        request=BlogCommentLikedCommentsRequestSerializer,
        responses={
            200: BlogCommentLikedCommentsResponseSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["POST"])
    def liked_comments(self, request, *args, **kwargs):
        serializer = BlogCommentLikedCommentsRequestSerializer(
            data=request.data
        )
        serializer.is_valid(raise_exception=True)

        comment_ids = serializer.validated_data["comment_ids"]
        user = request.user

        liked_ids = list(
            BlogComment.objects.filter(
                id__in=comment_ids, likes=user
            ).values_list("id", flat=True)
        )

        response_serializer = BlogCommentLikedCommentsResponseSerializer(
            {"liked_comment_ids": liked_ids}
        )
        return Response(response_serializer.data)

    @extend_schema(
        operation_id="listMyBlogComments",
        summary=_("Get current user's comments"),
        description=_(
            "Get all comments made by the currently authenticated user."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
            401: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["GET"])
    def my_comments(self, request):
        queryset = self.get_queryset().filter(user=request.user)
        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="getMyBlogComment",
        summary=_("Get current user's comment"),
        description=_(
            "Get the comment made by the currently authenticated user for a specific post."
        ),
        tags=["Blog Comments"],
        request=BlogCommentMyCommentRequestSerializer,
        responses={
            200: BlogCommentDetailSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["POST"])
    def my_comment(self, request, *args, **kwargs) -> Response:
        serializer = BlogCommentMyCommentRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        post_id = serializer.validated_data["post"]

        try:
            post = BlogPost.objects.get(id=post_id)
        except BlogPost.DoesNotExist:
            return Response(
                {"error": "Post not found"}, status=status.HTTP_404_NOT_FOUND
            )

        try:
            comment = self.get_queryset().get(user=request.user, post=post)
            serializer = self.get_serializer(comment)
            return Response(serializer.data)
        except BlogComment.DoesNotExist:
            return Response(
                {"error": "Comment not found for this post"},
                status=status.HTTP_404_NOT_FOUND,
            )



from __future__ import annotations

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.response import Response

from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from product.filters.product import ProductFilter
from product.models.product import Product
from product.serializers.image import ProductImageSerializer
from product.serializers.product import (
    ProductDetailSerializer,
    ProductSerializer,
    ProductWriteSerializer,
)
from product.serializers.review import ProductReviewSerializer
from tag.serializers.tag import TagSerializer

schema_config = create_schema_view_config(
    model_class=Product,
    display_config={
        "tag": "Products",
    },
    serializers={
        "list_serializer": ProductSerializer,
        "detail_serializer": ProductDetailSerializer,
        "write_serializer": ProductWriteSerializer,
    },
)


@extend_schema_view(**schema_config)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class ProductViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = Product.objects.all()
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = ProductFilter
    ordering_fields = [
        "price",
        "created_at",
        "discount_value_amount",
        "final_price_amount",
        "price_save_percent_field",
        "review_average_field",
        "likes_count_field",
        "view_count",
        "stock",
    ]
    ordering = ["-created_at"]
    search_fields = ["translations__name", "translations__description", "slug"]

    serializers = {
        "default": ProductDetailSerializer,
        "list": ProductSerializer,
        "retrieve": ProductDetailSerializer,
        "create": ProductWriteSerializer,
        "update": ProductWriteSerializer,
        "partial_update": ProductWriteSerializer,
        "update_view_count": ProductDetailSerializer,
        "reviews": ProductReviewSerializer,
        "images": ProductImageSerializer,
        "tags": TagSerializer,
    }

    response_serializers = {
        "create": ProductDetailSerializer,
        "update": ProductDetailSerializer,
        "partial_update": ProductDetailSerializer,
    }

    def get_queryset(self):
        queryset = super().get_queryset()
        return queryset.with_all_annotations()

    @extend_schema(
        operation_id="incrementProductViews",
        summary=_("Increment product view count"),
        description=_("Increment the view count for a product."),
        tags=["Products"],
        responses={
            200: ProductDetailSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["POST"],
    )
    def update_view_count(self, request, pk=None):
        product = self.get_object()
        product.view_count += 1
        product.save()
        serializer = ProductDetailSerializer(
            product, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listProductReviews",
        summary=_("Get product reviews"),
        description=_("Get all reviews for a product."),
        tags=["Products"],
        responses={
            200: ProductReviewSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def reviews(self, request, pk=None):
        product = self.get_object()
        reviews = product.reviews.all()
        serializer = ProductReviewSerializer(
            reviews, many=True, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listProductImages",
        summary=_("Get product images"),
        description=_("Get all images for a product."),
        tags=["Products"],
        responses={
            200: ProductImageSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def images(self, request, pk=None):
        product = self.get_object()
        images = product.images.all()
        serializer = ProductImageSerializer(
            images, many=True, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listProductTags",
        summary=_("Get product tags"),
        description=_("Get all tags associated with a product."),
        tags=["Products"],
        parameters=[],
        responses={
            200: TagSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def tags(self, request, pk=None):
        product = self.get_object()
        tags = [tagged_item.tag for tagged_item in product.tags.all()]
        serializer = TagSerializer(
            tags, many=True, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)



from django.conf import settings


def generate_schema_multi_lang(model_instance):
    fields = {}
    translated_fields = model_instance._parler_meta.get_all_fields()
    languages = settings.PARLER_LANGUAGES[settings.SITE_ID]

    if not translated_fields or not languages:
        return {"type": "object", "properties": {}}

    for language in languages:
        fields[language["code"]] = {"type": "object", "properties": {}}
        for translated_field in translated_fields:
            fields[language["code"]]["properties"][translated_field] = {
                "type": "string"
            }
    return {"type": "object", "properties": fields}



from django.db import models
from django.db.models import Count
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from blog.models.author import BlogAuthor


class BlogAuthorFilter(filters.FilterSet):
    user = filters.NumberFilter(
        field_name="user__id",
        lookup_expr="exact",
        help_text=_("Filter by user ID"),
    )
    user_email = filters.CharFilter(
        field_name="user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )
    first_name = filters.CharFilter(
        field_name="user__first_name",
        lookup_expr="icontains",
        help_text=_("Filter by first name (partial match)"),
    )
    last_name = filters.CharFilter(
        field_name="user__last_name",
        lookup_expr="icontains",
        help_text=_("Filter by last name (partial match)"),
    )
    full_name = filters.CharFilter(
        method="filter_full_name",
        help_text=_("Filter by full name (first + last name)"),
    )

    has_website = filters.BooleanFilter(
        method="filter_has_website",
        help_text=_("Filter authors who have/don't have a website"),
    )
    website = filters.CharFilter(
        field_name="website",
        lookup_expr="icontains",
        help_text=_("Filter by website URL (partial match)"),
    )

    bio = filters.CharFilter(
        field_name="translations__bio",
        lookup_expr="icontains",
        help_text=_("Filter by bio content (partial match)"),
    )

    min_posts = filters.NumberFilter(
        method="filter_min_posts",
        help_text=_("Filter authors with at least X posts"),
    )
    max_posts = filters.NumberFilter(
        method="filter_max_posts",
        help_text=_("Filter authors with at most X posts"),
    )
    has_posts = filters.BooleanFilter(
        method="filter_has_posts",
        help_text=_("Filter authors who have/don't have posts"),
    )

    min_total_likes = filters.NumberFilter(
        method="filter_min_total_likes",
        help_text=_(
            "Filter authors with at least X total likes across all posts"
        ),
    )
    has_likes = filters.BooleanFilter(
        method="filter_has_likes",
        help_text=_("Filter authors who have/don't have likes on their posts"),
    )

    most_active = filters.BooleanFilter(
        method="filter_most_active",
        help_text=_("Get most active authors (top 10 by post count)"),
    )
    most_liked = filters.BooleanFilter(
        method="filter_most_liked",
        help_text=_("Get most liked authors (top 10 by total likes)"),
    )

    class Meta:
        model = BlogAuthor
        fields = [
            "id",
            "user",
            "user_email",
            "first_name",
            "last_name",
            "full_name",
            "has_website",
            "website",
            "bio",
            "min_posts",
            "max_posts",
            "has_posts",
            "min_total_likes",
            "has_likes",
            "most_active",
            "most_liked",
        ]

    def filter_full_name(self, queryset, name, value):
        if not value:
            return queryset

        names = value.strip().split()
        if len(names) == 1:
            return queryset.filter(
                models.Q(user__first_name__icontains=names[0])
                | models.Q(user__last_name__icontains=names[0])
            )
        elif len(names) >= 2:
            return queryset.filter(
                user__first_name__icontains=names[0],
                user__last_name__icontains=names[-1],
            )
        return queryset

    def filter_has_website(self, queryset, name, value):
        if value is True:
            return queryset.exclude(website__isnull=True).exclude(
                website__exact=""
            )
        elif value is False:
            return queryset.filter(
                models.Q(website__isnull=True) | models.Q(website__exact="")
            )
        return queryset

    def filter_min_posts(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__gte=value
            )
        return queryset

    def filter_max_posts(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__lte=value
            )
        return queryset

    def filter_has_posts(self, queryset, name, value):
        if value is True:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__gt=0
            )
        elif value is False:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count=0
            )
        return queryset

    def filter_min_total_likes(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(
                total_likes=Count("blog_posts__likes")
            ).filter(total_likes__gte=value)
        return queryset

    def filter_has_likes(self, queryset, name, value):
        if value is True:
            return queryset.annotate(
                total_likes=Count("blog_posts__likes")
            ).filter(total_likes__gt=0)
        elif value is False:
            return queryset.annotate(
                total_likes=Count("blog_posts__likes")
            ).filter(total_likes=0)
        return queryset

    def filter_most_active(self, queryset, name, value):
        if value is True:
            return (
                queryset.annotate(post_count=Count("blog_posts"))
                .filter(post_count__gt=0)
                .order_by("-post_count")[:10]
            )
        return queryset

    def filter_most_liked(self, queryset, name, value):
        if value is True:
            return (
                queryset.annotate(total_likes=Count("blog_posts__likes"))
                .filter(total_likes__gt=0)
                .order_by("-total_likes")[:10]
            )
        return queryset



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from country.models import Country


class CountryFilter(filters.FilterSet):
    id = filters.UUIDFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by country UUID"),
    )
    alpha_2 = filters.CharFilter(
        field_name="alpha_2",
        lookup_expr="icontains",
        help_text=_("Filter by 2-letter country code (partial match)"),
    )
    alpha_3 = filters.CharFilter(
        field_name="alpha_3",
        lookup_expr="icontains",
        help_text=_("Filter by 3-letter country code (partial match)"),
    )
    name = filters.CharFilter(
        field_name="translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by country name (partial match)"),
    )
    iso_cc = filters.NumberFilter(
        field_name="iso_cc",
        lookup_expr="exact",
        help_text=_("Filter by ISO country code"),
    )
    phone_code = filters.NumberFilter(
        field_name="phone_code",
        lookup_expr="exact",
        help_text=_("Filter by phone code"),
    )

    iso_cc_min = filters.NumberFilter(
        field_name="iso_cc",
        lookup_expr="gte",
        help_text=_("Filter countries with ISO code greater than or equal to"),
    )
    iso_cc_max = filters.NumberFilter(
        field_name="iso_cc",
        lookup_expr="lte",
        help_text=_("Filter countries with ISO code less than or equal to"),
    )
    phone_code_min = filters.NumberFilter(
        field_name="phone_code",
        lookup_expr="gte",
        help_text=_(
            "Filter countries with phone code greater than or equal to"
        ),
    )
    phone_code_max = filters.NumberFilter(
        field_name="phone_code",
        lookup_expr="lte",
        help_text=_("Filter countries with phone code less than or equal to"),
    )

    has_iso_cc = filters.BooleanFilter(
        method="filter_has_iso_cc",
        help_text=_("Filter countries that have ISO country code"),
    )
    has_phone_code = filters.BooleanFilter(
        method="filter_has_phone_code",
        help_text=_("Filter countries that have phone code"),
    )
    has_flag_image = filters.BooleanFilter(
        method="filter_has_flag_image",
        help_text=_("Filter countries that have flag image"),
    )

    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter countries created after this date"),
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter countries created before this date"),
    )

    class Meta:
        model = Country
        fields = [
            "id",
            "alpha_2",
            "alpha_3",
            "name",
            "iso_cc",
            "phone_code",
            "iso_cc_min",
            "iso_cc_max",
            "phone_code_min",
            "phone_code_max",
            "has_iso_cc",
            "has_phone_code",
            "has_flag_image",
            "created_after",
            "created_before",
        ]

    def filter_has_iso_cc(self, queryset, name, value):
        if value is not None:
            if value:
                return queryset.filter(iso_cc__isnull=False)
            else:
                return queryset.filter(iso_cc__isnull=True)
        return queryset

    def filter_has_phone_code(self, queryset, name, value):
        if value is not None:
            if value:
                return queryset.filter(phone_code__isnull=False)
            else:
                return queryset.filter(phone_code__isnull=True)
        return queryset

    def filter_has_flag_image(self, queryset, name, value):
        if value is not None:
            if value:
                return queryset.exclude(image_flag__isnull=True).exclude(
                    image_flag=""
                )
            else:
                return queryset.filter(
                    models.Q(image_flag__isnull=True) | models.Q(image_flag="")
                )
        return queryset



from django.conf import settings
from django.contrib.auth import get_user_model
from phonenumber_field.serializerfields import PhoneNumberField
from rest_framework import serializers

User = get_user_model()


class UserDetailsSerializer(serializers.ModelSerializer[User]):
    def validate_username(self, username: str) -> str:
        if "allauth.account" not in settings.INSTALLED_APPS:
            return username

        from allauth.account.adapter import get_adapter  # noqa: PLC0415

        username = get_adapter().clean_username(username)
        return username

    class Meta:
        extra_fields = []
        if hasattr(User, "USERNAME_FIELD"):
            extra_fields.append(User.USERNAME_FIELD)
        if hasattr(User, "EMAIL_FIELD"):
            extra_fields.append(User.EMAIL_FIELD)
        if hasattr(User, "first_name"):
            extra_fields.append("first_name")
        if hasattr(User, "last_name"):
            extra_fields.append("last_name")
        model = User
        fields = ("pk", *extra_fields)
        read_only_fields = ("email",)


class AuthenticationSerializer(UserDetailsSerializer):
    phone = PhoneNumberField(required=False)

    class Meta(UserDetailsSerializer.Meta):
        fields = (
            *UserDetailsSerializer.Meta.fields,
            "id",
            "email",
            "username",
            "image",
            "first_name",
            "last_name",
            "phone",
            "city",
            "zipcode",
            "address",
            "place",
            "country",
            "region",
            "birth_date",
            "twitter",
            "linkedin",
            "facebook",
            "instagram",
            "website",
            "youtube",
            "github",
            "bio",
            "is_active",
            "is_staff",
            "is_superuser",
            "created_at",
            "updated_at",
            "uuid",
            "main_image_path",
        )
        read_only_fields = (
            "is_active",
            "is_staff",
            "is_superuser",
            "created_at",
            "updated_at",
            "uuid",
            "full_name",
            "main_image_path",
        )



from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin as BaseGroupAdmin
from django.contrib.auth.models import Group
from django.db import models
from django.db.models import Count, Q
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from parler.admin import TranslatableAdmin
from unfold.admin import ModelAdmin, TabularInline
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateFilter,
    RangeDateTimeFilter,
    RangeNumericListFilter,
    RelatedDropdownFilter,
)
from unfold.contrib.forms.widgets import WysiwygWidget
from unfold.decorators import action
from unfold.enums import ActionVariant
from unfold.forms import (
    AdminPasswordChangeForm,
    UserChangeForm,
    UserCreationForm,
)

from user.models import UserAccount
from user.models.address import UserAddress
from user.models.subscription import SubscriptionTopic, UserSubscription

admin.site.unregister(Group)


class SubscriptionCountFilter(RangeNumericListFilter):
    title = _("Subscription Count")
    parameter_name = "subscription_count"

    def queryset(self, request, queryset):
        filters = {}

        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["subscription_count__gte"] = value_from

        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["subscription_count__lte"] = value_to

        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class AddressCountFilter(RangeNumericListFilter):
    title = _("Address Count")
    parameter_name = "address_count"

    def queryset(self, request, queryset):
        filters = {}

        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["address_count__gte"] = value_from

        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["address_count__lte"] = value_to

        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class UserStatusFilter(DropdownFilter):
    title = _("User Status")
    parameter_name = "user_status"

    def lookups(self, request, model_admin):
        return [
            ("active_staff", _("Active Staff")),
            ("active_regular", _("Active Regular Users")),
            ("inactive", _("Inactive Users")),
            ("superuser", _("Super Users")),
            ("with_subscriptions", _("Users with Subscriptions")),
            ("no_subscriptions", _("Users without Subscriptions")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "active_staff":
                filter_kwargs = {"is_active": True, "is_staff": True}
            case "active_regular":
                filter_kwargs = {
                    "is_active": True,
                    "is_staff": False,
                    "is_superuser": False,
                }
            case "inactive":
                filter_kwargs = {"is_active": False}
            case "superuser":
                filter_kwargs = {"is_superuser": True}
            case "with_subscriptions":
                return queryset.filter(subscriptions__isnull=False).distinct()
            case "no_subscriptions":
                filter_kwargs = {"subscriptions__isnull": True}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class SocialMediaFilter(DropdownFilter):
    title = _("Social Media Presence")
    parameter_name = "social_media"

    def lookups(self, request, model_admin):
        return [
            ("has_website", _("Has Website")),
            ("has_social", _("Has Social Media")),
            ("no_social", _("No Social Media")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "has_website":
                return queryset.exclude(website="")
            case "has_social":
                return queryset.filter(
                    Q(twitter__gt="")
                    | Q(linkedin__gt="")
                    | Q(facebook__gt="")
                    | Q(instagram__gt="")
                    | Q(youtube__gt="")
                    | Q(github__gt="")
                )
            case "no_social":
                filter_kwargs = {
                    "website": "",
                    "twitter": "",
                    "linkedin": "",
                    "facebook": "",
                    "instagram": "",
                    "youtube": "",
                    "github": "",
                }
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class TopicCategoryFilter(DropdownFilter):
    title = _("Topic Category")
    parameter_name = "topic_category"

    def lookups(self, request, model_admin):
        return SubscriptionTopic.TopicCategory.choices

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(topic__category=self.value())
        return queryset


class UserAddressInline(TabularInline):
    model = UserAddress
    extra = 0
    fields = (
        "title",
        "first_name",
        "last_name",
        "city",
        "country",
        "is_main",
        "phone",
    )
    readonly_fields = ("created_at",)
    show_change_link = True
    tab = True


class UserSubscriptionInline(TabularInline):
    model = UserSubscription
    extra = 0
    fields = ("topic", "status", "subscribed_at", "unsubscribed_at")
    readonly_fields = ("subscribed_at", "unsubscribed_at")
    show_change_link = True
    tab = True


@admin.register(Group)
class GroupAdmin(BaseGroupAdmin, ModelAdmin):
    pass


@admin.register(UserAccount)
class UserAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    form = UserChangeForm
    add_form = UserCreationForm
    change_password_form = AdminPasswordChangeForm

    formfield_overrides = {
        models.TextField: {
            "widget": WysiwygWidget,
        },
    }

    list_display = [
        "id",
        "user_profile_display",
        "contact_info_display",
        "location_display",
        "user_status_badges",
        "social_links_display",
        "engagement_metrics",
        "last_activity",
        "created_at",
    ]

    list_filter = [
        UserStatusFilter,
        SocialMediaFilter,
        "is_active",
        "is_staff",
        "is_superuser",
        ("country", RelatedDropdownFilter),
        ("region", RelatedDropdownFilter),
        SubscriptionCountFilter,
        AddressCountFilter,
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
        ("birth_date", RangeDateFilter),
    ]

    search_fields = [
        "email",
        "username",
        "first_name",
        "last_name",
        "phone",
        "city",
        "address",
        "bio",
    ]

    list_select_related = ["country", "region"]
    readonly_fields = [
        "id",
        "created_at",
        "updated_at",
        "engagement_metrics",
        "social_links_summary",
        "subscription_summary",
        "address_summary",
    ]

    ordering = ["-created_at"]
    date_hierarchy = "created_at"
    save_on_top = True
    list_per_page = 25

    fieldsets = (
        (
            _("Account Credentials"),
            {
                "fields": ("email", "username", "password"),
                "classes": ("wide",),
            },
        ),
        (
            _("Personal Information"),
            {
                "fields": (
                    "first_name",
                    "last_name",
                    "phone",
                    "birth_date",
                    "image",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Bio & Description"),
            {
                "fields": ("bio",),
                "classes": ("wide",),
            },
        ),
        (
            _("Location & Address"),
            {
                "fields": (
                    "address",
                    "city",
                    "zipcode",
                    "place",
                    "country",
                    "region",
                    "address_summary",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Social Media & Website"),
            {
                "fields": (
                    "website",
                    "linkedin",
                    "github",
                    "twitter",
                    "facebook",
                    "instagram",
                    "youtube",
                    "social_links_summary",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Account Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Subscriptions & Engagement"),
            {
                "fields": ("subscription_summary", "engagement_metrics"),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("email", "username", "password1", "password2"),
            },
        ),
    )

    inlines = [UserAddressInline, UserSubscriptionInline]

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        qs = qs.annotate(
            subscription_count=Count("subscriptions", distinct=True),
            address_count=Count("addresses", distinct=True),
            active_subscription_count=Count(
                "subscriptions",
                filter=Q(
                    subscriptions__status=UserSubscription.SubscriptionStatus.ACTIVE
                ),
                distinct=True,
            ),
        )
        return qs

    def user_profile_display(self, obj):
        profile_html = '<div class="flex items-center gap-3">'

        if obj.image:
            profile_html += f'<img src="{obj.image.url}" class="rounded-full object-cover" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;" />'
        else:
            initials = "".join(
                [
                    name[0].upper()
                    for name in [obj.first_name, obj.last_name]
                    if name
                ]
            )[:2]
            if not initials:
                initials = obj.email[0].upper() if obj.email else "U"
            profile_html += f'<div class="rounded-full bg-blue-100 dark:bg-blue-900 flex items-center justify-center text-blue-600 dark:text-blue-300 font-medium text-sm" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;">{initials}</div>'

        full_name = obj.full_name or obj.username or "Anonymous"
        profile_html += "<div>"
        profile_html += f'<div class="font-medium text-base-900 dark:text-base-100">{full_name}</div>'
        profile_html += f'<div class="text-sm text-base-500 dark:text-base-400">{obj.email}</div>'
        profile_html += "</div></div>"

        return format_html(profile_html)

    user_profile_display.short_description = _("Profile")

    def contact_info_display(self, obj):
        contact_parts = []
        if obj.phone:
            contact_parts.append(
                f'<span class="flex items-center gap-1"><span>📞</span><span>{obj.phone}</span></span>'
            )
        if obj.username:
            contact_parts.append(
                f'<span class="flex items-center gap-1"><span>👤</span><span>{obj.username}</span></span>'
            )

        if contact_parts:
            return mark_safe(
                f'<div class="text-sm text-base-700 dark:text-base-300 space-y-1">{"".join(contact_parts)}</div>'
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500">No contact info</span>'
        )

    contact_info_display.short_description = _("Contact")

    def location_display(self, obj):
        location_parts = []
        if obj.city:
            location_parts.append(obj.city)
        if obj.country:
            location_parts.append(str(obj.country))

        if location_parts:
            return format_html(
                '<div class="text-sm text-base-700 dark:text-base-300">'
                '<span class="flex items-center gap-1">'
                "<span>📍</span><span>{}</span>"
                "</span></div>",
                ", ".join(location_parts),
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500">No location</span>'
        )

    location_display.short_description = _("Location")

    def user_status_badges(self, obj):
        badges = []

        if obj.is_superuser:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
                "<span>👑</span><span>Super</span></span>"
            )
        elif obj.is_staff:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300 rounded-full gap-1">'
                "<span>⚡</span><span>Staff</span></span>"
            )

        if obj.is_active:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full gap-1">'
                "<span>✓</span><span>Active</span></span>"
            )
        else:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
                "<span>✗</span><span>Inactive</span></span>"
            )

        return mark_safe(
            f'<div class="flex flex-wrap gap-1">{"".join(badges)}</div>'
        )

    user_status_badges.short_description = _("Status")

    def social_links_display(self, obj):
        links = []
        social_fields = {
            "website": ("🌐", "Website"),
            "linkedin": ("💼", "LinkedIn"),
            "github": ("💻", "GitHub"),
            "twitter": ("🐦", "Twitter"),
        }

        for field, (icon, name) in social_fields.items():
            if getattr(obj, field):
                links.append(f'<span title="{name}">{icon}</span>')

        if links:
            return mark_safe(f'<div class="flex gap-1">{"".join(links)}</div>')
        return format_html(
            '<span class="text-base-400 dark:text-base-500">-</span>'
        )

    social_links_display.short_description = _("Social")

    def engagement_metrics(self, obj):
        subscriptions = getattr(obj, "subscription_count", 0)
        active_subs = getattr(obj, "active_subscription_count", 0)
        addresses = getattr(obj, "address_count", 0)

        return format_html(
            '<div class="text-sm text-base-700 dark:text-base-300 flex items-center gap-3">'
            '<span class="flex items-center gap-1 text-blue-600 dark:text-blue-400">'
            "<span>📧</span><span>{}/{}</span>"
            "</span>"
            '<span class="flex items-center gap-1 text-green-600 dark:text-green-400">'
            "<span>📍</span><span>{}</span>"
            "</span>"
            "</div>",
            active_subs,
            subscriptions,
            addresses,
        )

    engagement_metrics.short_description = _("Engagement")

    def last_activity(self, obj):
        return format_html(
            '<div class="text-sm text-base-600 dark:text-base-400">'
            "<div>Updated: {}</div>"
            "</div>",
            obj.updated_at.strftime("%Y-%m-%d %H:%M")
            if obj.updated_at
            else "Never",
        )

    last_activity.short_description = _("Last Activity")

    def social_links_summary(self, obj):
        links = []
        social_fields = {
            "website": "Website",
            "linkedin": "LinkedIn",
            "github": "GitHub",
            "twitter": "Twitter",
            "facebook": "Facebook",
            "instagram": "Instagram",
            "youtube": "YouTube",
        }

        for field, name in social_fields.items():
            value = getattr(obj, field)
            if value:
                links.append(
                    f'<a href="{value}" target="_blank" class="text-blue-600 dark:text-blue-400 hover:underline">{name}</a>'
                )

        if links:
            return mark_safe(
                f'<div class="space-y-1">{"<br>".join(links)}</div>'
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500 italic">No social media links</span>'
        )

    social_links_summary.short_description = _("Social Links Summary")

    def subscription_summary(self, obj):
        subscriptions = obj.subscriptions.select_related("topic").all()
        if not subscriptions:
            return format_html(
                '<span class="text-base-400 dark:text-base-500 italic">No subscriptions</span>'
            )

        active_count = sum(
            1
            for sub in subscriptions
            if sub.status == UserSubscription.SubscriptionStatus.ACTIVE
        )
        total_count = len(subscriptions)

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">Active: {}/{}</div>'
            "</div>",
            active_count,
            total_count,
        )

    subscription_summary.short_description = _("Subscription Summary")

    def address_summary(self, obj):
        addresses = obj.addresses.all()
        if not addresses:
            return format_html(
                '<span class="text-base-400 dark:text-base-500 italic">No addresses</span>'
            )

        main_address = next((addr for addr in addresses if addr.is_main), None)

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">Total: {}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            len(addresses),
            f"Main: {main_address}" if main_address else "No main address",
        )

    address_summary.short_description = _("Address Summary")


@admin.register(UserAddress)
class UserAddressAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "id",
        "address_display",
        "contact_person",
        "location_info",
        "main_address_badge",
        "contact_numbers",
        "created_at",
    ]

    list_filter = [
        "is_main",
        "floor",
        "location_type",
        ("country", RelatedDropdownFilter),
        ("region", RelatedDropdownFilter),
        ("created_at", RangeDateTimeFilter),
    ]

    search_fields = [
        "user__email",
        "user__username",
        "title",
        "first_name",
        "last_name",
        "street",
        "city",
    ]

    list_select_related = ["user", "country", "region"]
    readonly_fields = ["id", "created_at", "updated_at"]

    fieldsets = (
        (
            _("Address Owner"),
            {
                "fields": ("user", "title"),
                "classes": ("wide",),
            },
        ),
        (
            _("Contact Person"),
            {
                "fields": ("first_name", "last_name"),
                "classes": ("wide",),
            },
        ),
        (
            _("Address Details"),
            {
                "fields": (
                    "street",
                    "street_number",
                    "city",
                    "zipcode",
                    "country",
                    "region",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Additional Information"),
            {
                "fields": (
                    "floor",
                    "location_type",
                    "notes",
                    "is_main",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Contact Information"),
            {
                "fields": ("phone", "mobile_phone"),
                "classes": ("wide",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def address_display(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            obj.title,
            f"{obj.street} {obj.street_number}, {obj.city}",
        )

    address_display.short_description = _("Address")

    def contact_person(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">{} {}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            obj.first_name,
            obj.last_name,
            obj.user.email,
        )

    contact_person.short_description = _("Contact Person")

    def location_info(self, obj):
        location_parts = [obj.city]
        if obj.country:
            location_parts.append(str(obj.country))
        if obj.zipcode:
            location_parts.append(obj.zipcode)

        return format_html(
            '<div class="text-sm text-base-700 dark:text-base-300">{}</div>',
            ", ".join(location_parts),
        )

    location_info.short_description = _("Location")

    def main_address_badge(self, obj):
        if obj.is_main:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full gap-1">'
                "<span>⭐</span><span>Main</span>"
                "</span>"
            )
        return ""

    main_address_badge.short_description = _("Main")

    def contact_numbers(self, obj):
        numbers = []
        if obj.phone:
            numbers.append(
                f'<span class="flex items-center gap-1"><span>📞</span><span>{obj.phone}</span></span>'
            )
        if obj.mobile_phone:
            numbers.append(
                f'<span class="flex items-center gap-1"><span>📱</span><span>{obj.mobile_phone}</span></span>'
            )

        if numbers:
            return mark_safe(
                f'<div class="text-sm text-base-700 dark:text-base-300 space-y-1">{"".join(numbers)}</div>'
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500">No phone</span>'
        )

    contact_numbers.short_description = _("Contact Numbers")


@admin.register(SubscriptionTopic)
class SubscriptionTopicAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = False
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "name_display",
        "category_badge",
        "active_status",
        "is_active",
        "settings_badges",
        "subscriber_metrics",
        "created_at",
    ]

    list_filter = [
        "category",
        "is_active",
        "is_default",
        "requires_confirmation",
        ("created_at", RangeDateTimeFilter),
    ]

    search_fields = ["translations__name", "slug", "translations__description"]
    readonly_fields = ["uuid", "created_at", "updated_at", "subscriber_metrics"]

    fieldsets = (
        (
            _("Topic Information"),
            {
                "fields": ("name", "slug", "description", "category"),
                "classes": ("wide",),
            },
        ),
        (
            _("Settings"),
            {
                "fields": (
                    "is_active",
                    "is_default",
                    "requires_confirmation",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": (
                    "uuid",
                    "created_at",
                    "updated_at",
                    "subscriber_metrics",
                ),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        qs = qs.annotate(
            active_subscribers=Count(
                "subscribers",
                filter=Q(
                    subscribers__status=UserSubscription.SubscriptionStatus.ACTIVE
                ),
            ),
            total_subscribers=Count("subscribers"),
        )
        return qs

    def name_display(self, obj):
        name = (
            obj.safe_translation_getter("name", any_language=True)
            or "Unnamed Topic"
        )
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            name,
            obj.slug,
        )

    name_display.short_description = _("Topic")

    def category_badge(self, obj):
        colors = {
            "MARKETING": "bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300",
            "PRODUCT": "bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300",
            "ACCOUNT": "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300",
            "SYSTEM": "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300",
            "NEWSLETTER": "bg-indigo-50 dark:bg-indigo-900 text-indigo-700 dark:text-indigo-300",
            "OTHER": "bg-base-50 dark:bg-base-900 text-base-700 dark:text-base-300",
        }
        color_class = colors.get(obj.category, colors["OTHER"])

        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} rounded-full">{}</span>',
            color_class,
            obj.get_category_display(),
        )

    category_badge.short_description = _("Category")

    def active_status(self, obj):
        if obj.is_active:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full gap-1">'
                "<span>✓</span><span>Active</span>"
                "</span>"
            )
        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
            "<span>✗</span><span>Inactive</span>"
            "</span>"
        )

    active_status.short_description = _("Status")

    def settings_badges(self, obj):
        badges = []
        if obj.is_default:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full gap-1">'
                "<span>⭐</span><span>Default</span></span>"
            )
        if obj.requires_confirmation:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 rounded-full gap-1">'
                "<span>✉️</span><span>Confirm</span></span>"
            )

        return (
            mark_safe(
                f'<div class="flex flex-wrap gap-1">{"".join(badges)}</div>'
            )
            if badges
            else ""
        )

    settings_badges.short_description = _("Settings")

    def subscriber_metrics(self, obj):
        active = getattr(obj, "active_subscribers", 0)
        total = getattr(obj, "total_subscribers", 0)

        return format_html(
            '<div class="text-sm text-base-700 dark:text-base-300 flex items-center gap-3">'
            '<span class="flex items-center gap-1 text-green-600 dark:text-green-400">'
            "<span>✓</span><span>{}</span>"
            "</span>"
            '<span class="flex items-center gap-1 text-blue-600 dark:text-blue-400">'
            "<span>👥</span><span>{}</span>"
            "</span>"
            "</div>",
            active,
            total,
        )

    subscriber_metrics.short_description = _("Subscribers")


@admin.register(UserSubscription)
class UserSubscriptionAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "subscription_info",
        "user_info",
        "topic_info",
        "status_display",
        "status",
        "subscription_dates",
        "created_at",
    ]

    list_filter = [
        "status",
        TopicCategoryFilter,
        ("topic", RelatedDropdownFilter),
        ("subscribed_at", RangeDateTimeFilter),
        ("unsubscribed_at", RangeDateTimeFilter),
    ]

    search_fields = [
        "user__email",
        "user__username",
        "topic__translations__name",
        "user__first_name",
        "user__last_name",
    ]

    readonly_fields = [
        "subscribed_at",
        "unsubscribed_at",
        "created_at",
        "updated_at",
    ]

    raw_id_fields = ["user"]
    autocomplete_fields = ["topic"]
    list_select_related = ["user", "topic"]

    actions = [
        "activate_subscriptions",
        "deactivate_subscriptions",
    ]

    fieldsets = (
        (
            _("Subscription Details"),
            {
                "fields": ("user", "topic", "status"),
                "classes": ("wide",),
            },
        ),
        (
            _("Timestamps"),
            {
                "fields": (
                    "subscribed_at",
                    "unsubscribed_at",
                    "created_at",
                    "updated_at",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Additional Data"),
            {
                "fields": ("confirmation_token", "metadata"),
                "classes": ("collapse",),
            },
        ),
    )

    def subscription_info(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">Subscription #{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            obj.id,
            obj.created_at.strftime("%Y-%m-%d"),
        )

    subscription_info.short_description = _("Subscription")

    def user_info(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            obj.user.full_name or obj.user.username or "Anonymous",
            obj.user.email,
        )

    user_info.short_description = _("User")

    def topic_info(self, obj):
        topic_name = (
            obj.topic.safe_translation_getter("name", any_language=True)
            or "Unnamed Topic"
        )
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            topic_name,
            obj.topic.get_category_display(),
        )

    topic_info.short_description = _("Topic")

    def status_display(self, obj):
        colors = {
            UserSubscription.SubscriptionStatus.ACTIVE: "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300",
            UserSubscription.SubscriptionStatus.PENDING: "bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300",
            UserSubscription.SubscriptionStatus.UNSUBSCRIBED: "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300",
            UserSubscription.SubscriptionStatus.BOUNCED: "bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300",
        }

        icons = {
            UserSubscription.SubscriptionStatus.ACTIVE: "✓",
            UserSubscription.SubscriptionStatus.PENDING: "⏳",
            UserSubscription.SubscriptionStatus.UNSUBSCRIBED: "✗",
            UserSubscription.SubscriptionStatus.BOUNCED: "⚠️",
        }

        color_class = colors.get(
            obj.status, colors[UserSubscription.SubscriptionStatus.ACTIVE]
        )
        icon = icons.get(obj.status, "?")

        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} rounded-full gap-1">'
            "<span>{}</span><span>{}</span>"
            "</span>",
            color_class,
            icon,
            obj.get_status_display(),
        )

    status_display.short_description = _("Status")

    def subscription_dates(self, obj):
        dates_html = '<div class="text-sm text-base-600 dark:text-base-400">'
        dates_html += f"<div>Subscribed: {obj.subscribed_at.strftime('%Y-%m-%d %H:%M')}</div>"
        if obj.unsubscribed_at:
            dates_html += f"<div>Unsubscribed: {obj.unsubscribed_at.strftime('%Y-%m-%d %H:%M')}</div>"
        dates_html += "</div>"

        return format_html(dates_html)

    subscription_dates.short_description = _("Dates")

    @action(
        description=_("Activate selected subscriptions"),
        variant=ActionVariant.SUCCESS,
        icon="check_circle",
    )
    def activate_subscriptions(self, request, queryset):
        updated = queryset.filter(
            status__in=[
                UserSubscription.SubscriptionStatus.PENDING,
                UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
            ]
        ).update(
            status=UserSubscription.SubscriptionStatus.ACTIVE,
            unsubscribed_at=None,
        )
        self.message_user(
            request,
            _("%(count)d subscriptions were activated.") % {"count": updated},
        )

    @action(
        description=_("Deactivate selected subscriptions"),
        variant=ActionVariant.WARNING,
        icon="cancel",
    )
    def deactivate_subscriptions(self, request, queryset):
        updated = queryset.filter(
            status=UserSubscription.SubscriptionStatus.ACTIVE
        ).update(
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
            unsubscribed_at=timezone.now(),
        )
        self.message_user(
            request,
            _("%(count)d subscriptions were deactivated.") % {"count": updated},
        )



import math

from rest_framework import pagination
from rest_framework.response import Response


class CursorPaginator(pagination.CursorPagination):
    page_size = 100
    max_page_size = 100
    page_size_query_param = "page_size"
    cursor_query_param = "cursor"
    total_items = 0
    total_pages = 0
    ordering = "-created_at"
    has_next = False
    has_previous = False

    def paginate_queryset(self, queryset, request, view=None):
        self.total_items = queryset.count()
        self.total_pages = self.get_total_pages()
        super().paginate_queryset(queryset, request, view)
        return self.page

    def get_total_pages(self):
        total_pages = math.ceil(self.total_items / self.page_size)
        if self.total_items % self.page_size != 0:
            total_pages += 1
        return total_pages

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.total_items,
                "total_pages": self.total_pages,
                "page_size": self.page_size,
                "page_total_results": len(data),
                "results": data,
            }
        )



from __future__ import annotations

from django.conf import settings
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema_view
from rest_framework.filters import SearchFilter

from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from product.models.category import ProductCategory
from product.serializers.category import (
    ProductCategoryDetailSerializer,
    ProductCategorySerializer,
    ProductCategoryWriteSerializer,
)


@extend_schema_view(
    **create_schema_view_config(
        model_class=ProductCategory,
        display_config={
            "tag": "Product Categories",
        },
        serializers={
            "list_serializer": ProductCategorySerializer,
            "detail_serializer": ProductCategoryDetailSerializer,
            "write_serializer": ProductCategoryWriteSerializer,
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class ProductCategoryViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = ProductCategory.objects.all()
    serializers = {
        "default": ProductCategoryDetailSerializer,
        "list": ProductCategorySerializer,
        "retrieve": ProductCategoryDetailSerializer,
        "create": ProductCategoryWriteSerializer,
        "update": ProductCategoryWriteSerializer,
        "partial_update": ProductCategoryWriteSerializer,
    }
    response_serializers = {
        "create": ProductCategoryDetailSerializer,
        "update": ProductCategoryDetailSerializer,
        "partial_update": ProductCategoryDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_fields = ["id", "parent", "slug"]
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
    ]
    ordering = ["-created_at"]
    search_fields = ["translations__name", "translations__description", "slug"]



import math

from rest_framework import pagination
from rest_framework.response import Response


class LimitOffsetPaginator(pagination.LimitOffsetPagination):
    max_limit = 100

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.count,
                "total_pages": math.ceil(self.count / self.limit),
                "page_size": self.limit,
                "page_total_results": len(data),
                "page": math.ceil(self.offset / self.limit) + 1,
                "results": data,
            }
        )



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from user.models.address import UserAddress


class UserAddressFilter(filters.FilterSet):
    location_type = filters.CharFilter(
        field_name="location_type",
        lookup_expr="iexact",
        help_text=_("Filter by location type (exact match, case insensitive)"),
    )
    location_type_contains = filters.CharFilter(
        field_name="location_type",
        lookup_expr="icontains",
        help_text="Filter by location type (partial match)",
    )

    country = filters.NumberFilter(
        field_name="country__id",
        lookup_expr="exact",
        help_text="Filter by country ID",
    )
    country_code = filters.CharFilter(
        field_name="country__code",
        lookup_expr="iexact",
        help_text="Filter by country code (e.g., 'US', 'CA')",
    )
    country_name = filters.CharFilter(
        field_name="country__name",
        lookup_expr="icontains",
        help_text="Filter by country name (partial match)",
    )

    region = filters.NumberFilter(
        field_name="region__id",
        lookup_expr="exact",
        help_text="Filter by region ID",
    )
    region_name = filters.CharFilter(
        field_name="region__name",
        lookup_expr="icontains",
        help_text="Filter by region name (partial match)",
    )

    city = filters.CharFilter(
        field_name="city",
        lookup_expr="icontains",
        help_text="Filter by city name (partial match)",
    )
    zipcode = filters.CharFilter(
        field_name="zipcode",
        lookup_expr="icontains",
        help_text="Filter by zipcode (partial match)",
    )
    zipcode_exact = filters.CharFilter(
        field_name="zipcode",
        lookup_expr="exact",
        help_text="Filter by exact zipcode",
    )

    street = filters.CharFilter(
        field_name="street",
        lookup_expr="icontains",
        help_text="Filter by street name (partial match)",
    )
    street_number = filters.CharFilter(
        field_name="street_number",
        lookup_expr="icontains",
        help_text="Filter by street number",
    )

    first_name = filters.CharFilter(
        field_name="first_name",
        lookup_expr="icontains",
        help_text="Filter by first name (partial match)",
    )
    last_name = filters.CharFilter(
        field_name="last_name",
        lookup_expr="icontains",
        help_text="Filter by last name (partial match)",
    )
    full_name = filters.CharFilter(
        method="filter_full_name",
        help_text="Filter by full name (first + last name)",
    )

    is_main = filters.BooleanFilter(
        field_name="is_main",
        lookup_expr="exact",
        help_text="Filter by main address status",
    )

    phone = filters.CharFilter(
        field_name="phone",
        lookup_expr="icontains",
        help_text="Filter by phone number (partial match)",
    )
    mobile_phone = filters.CharFilter(
        field_name="mobile_phone",
        lookup_expr="icontains",
        help_text="Filter by mobile phone number (partial match)",
    )

    created_after = filters.DateFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text="Filter addresses created after this date",
    )
    created_before = filters.DateFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text="Filter addresses created before this date",
    )
    updated_after = filters.DateFilter(
        field_name="updated_at",
        lookup_expr="gte",
        help_text="Filter addresses updated after this date",
    )
    updated_before = filters.DateFilter(
        field_name="updated_at",
        lookup_expr="lte",
        help_text="Filter addresses updated before this date",
    )

    class Meta:
        model = UserAddress
        fields = [
            "id",
            "title",
            "location_type",
            "location_type_contains",
            "country",
            "country_code",
            "country_name",
            "region",
            "region_name",
            "city",
            "zipcode",
            "zipcode_exact",
            "street",
            "street_number",
            "first_name",
            "last_name",
            "full_name",
            "is_main",
            "phone",
            "mobile_phone",
            "created_after",
            "created_before",
            "updated_after",
            "updated_before",
        ]

    def filter_full_name(self, queryset, name, value):
        if not value:
            return queryset

        names = value.strip().split()
        if len(names) == 1:
            return queryset.filter(
                models.Q(first_name__icontains=names[0])
                | models.Q(last_name__icontains=names[0])
            )
        elif len(names) >= 2:
            return queryset.filter(
                first_name__icontains=names[0], last_name__icontains=names[-1]
            )
        return queryset



from __future__ import annotations

from django.conf import settings
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema_view
from rest_framework.filters import SearchFilter

from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from country.filters import CountryFilter
from country.models import Country
from country.serializers import (
    CountryDetailSerializer,
    CountrySerializer,
    CountryWriteSerializer,
)


@extend_schema_view(
    **create_schema_view_config(
        model_class=Country,
        display_config={
            "tag": "Countries",
        },
        serializers={
            "list_serializer": CountrySerializer,
            "detail_serializer": CountryDetailSerializer,
            "write_serializer": CountryWriteSerializer,
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class CountryViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = Country.objects.all()
    serializers = {
        "default": CountryDetailSerializer,
        "list": CountrySerializer,
        "retrieve": CountryDetailSerializer,
        "create": CountryWriteSerializer,
        "update": CountryWriteSerializer,
        "partial_update": CountryWriteSerializer,
    }
    response_serializers = {
        "create": CountryDetailSerializer,
        "update": CountryDetailSerializer,
        "partial_update": CountryDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = CountryFilter
    ordering_fields = [
        "alpha_2",
        "alpha_3",
        "iso_cc",
        "phone_code",
        "created_at",
        "updated_at",
        "sort_order",
        "translations__name",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "alpha_2",
        "alpha_3",
        "iso_cc",
        "phone_code",
        "translations__name",
    ]



from datetime import timedelta

from django.contrib import admin, messages
from django.contrib.contenttypes.admin import GenericTabularInline
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils import timezone
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext
from parler.admin import TranslatableAdmin
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
)
from unfold.decorators import action
from unfold.enums import ActionVariant

from tag.models.tag import Tag
from tag.models.tagged_item import TaggedItem


class TagStatusFilter(DropdownFilter):
    title = _("Tag Status")
    parameter_name = "tag_status"

    def lookups(self, request, model_admin):
        return [
            ("active", _("Active Tags")),
            ("inactive", _("Inactive Tags")),
            ("used", _("Used Tags")),
            ("unused", _("Unused Tags")),
            ("popular", _("Popular (>10 uses)")),
            ("recent", _("Recently Created")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "active":
                filter_kwargs = {"active": True}
            case "inactive":
                filter_kwargs = {"active": False}
            case "used":
                return queryset.filter(taggeditem__isnull=False).distinct()
            case "unused":
                filter_kwargs = {"taggeditem__isnull": True}
            case "popular":
                return queryset.annotate(
                    usage_count=models.Count("taggeditem")
                ).filter(usage_count__gt=10)
            case "recent":
                seven_days_ago = timezone.now() - timedelta(days=7)
                filter_kwargs = {"created_at__gte": seven_days_ago}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class TagUsageFilter(DropdownFilter):
    title = _("Usage Range")
    parameter_name = "usage_range"

    def lookups(self, request, model_admin):
        return [
            ("single", _("Single Use (1)")),
            ("low", _("Low Usage (2-5)")),
            ("medium", _("Medium Usage (6-20)")),
            ("high", _("High Usage (21-50)")),
            ("very_high", _("Very High Usage (50+)")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        queryset = queryset.annotate(usage_count=models.Count("taggeditem"))

        match filter_value:
            case "single":
                filter_kwargs = {"usage_count": 1}
            case "low":
                filter_kwargs = {"usage_count__range": (2, 5)}
            case "medium":
                filter_kwargs = {"usage_count__range": (6, 20)}
            case "high":
                filter_kwargs = {"usage_count__range": (21, 50)}
            case "very_high":
                filter_kwargs = {"usage_count__gt": 50}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class ContentTypeFilter(DropdownFilter):
    title = _("Content Type")
    parameter_name = "content_type"

    def lookups(self, request, model_admin):
        content_types = (
            ContentType.objects.filter(taggeditem__isnull=False)
            .distinct()
            .values_list("id", "model")
        )

        return [(ct_id, ct_model.title()) for ct_id, ct_model in content_types]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(
                taggeditem__content_type_id=self.value()
            ).distinct()
        return queryset


class TagInLine(GenericTabularInline):
    model = TaggedItem
    autocomplete_fields = ["tag"]
    extra = 0
    fields = ("tag",)
    verbose_name = "Tag"
    verbose_name_plural = "Tags"

    classes = ["collapse"]


@admin.register(Tag)
class TagAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "tag_info",
        "status_badge",
        "usage_stats",
        "content_distribution",
        "sort_display",
        "created_display",
    ]
    list_filter = [
        TagStatusFilter,
        TagUsageFilter,
        ContentTypeFilter,
        "active",
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "translations__label",
        "id",
    ]
    readonly_fields = (
        "id",
        "uuid",
        "sort_order",
        "created_at",
        "updated_at",
        "tag_analytics",
        "usage_analytics",
        "content_analytics",
    )
    list_per_page = 50
    ordering = ["sort_order", "-id"]
    actions = [
        "activate_tags",
        "deactivate_tags",
        "update_sort_order",
        "analyze_usage",
    ]

    fieldsets = (
        (
            _("Tag Information"),
            {
                "fields": ("label", "active"),
                "classes": ("wide",),
            },
        ),
        (
            _("Organization"),
            {
                "fields": ("sort_order",),
                "classes": ("wide",),
            },
        ),
        (
            _("Analytics"),
            {
                "fields": (
                    "tag_analytics",
                    "usage_analytics",
                    "content_analytics",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .annotate(
                usage_count=models.Count("taggeditem"),
                content_types_count=models.Count(
                    "taggeditem__content_type", distinct=True
                ),
            )
            .prefetch_related("taggeditem_set__content_type")
        )

    def tag_info(self, obj):
        label = (
            obj.safe_translation_getter("label", any_language=True)
            or "Unnamed Tag"
        )

        label_display = label[:30] + "..." if len(label) > 30 else label

        usage_count = getattr(obj, "usage_count", 0)
        if usage_count > 0:
            usage_color = "text-green-600 dark:text-green-400"
            usage_icon = "🏷️"
        else:
            usage_color = "text-base-500 dark:text-base-400"
            usage_icon = "🏷️"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100 flex items-center gap-2">'
            '<span class="{}">{}</span>'
            "<span>{}</span>"
            "</div>"
            '<div class="text-base-600 dark:text-base-400">ID: {}</div>'
            '<div class="text-xs text-base-500 dark:text-base-400">Sort: {}</div>'
            "</div>",
            usage_color,
            usage_icon,
            label_display,
            obj.id,
            obj.sort_order or "No order",
        )

    tag_info.short_description = _("Tag")

    def status_badge(self, obj):
        if obj.active:
            status_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded-full">'
                "✅ Active"
                "</span>"
            )
        else:
            status_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 rounded-full">'
                "❌ Inactive"
                "</span>"
            )

        label = obj.safe_translation_getter("label", any_language=True) or ""
        label_length = len(label)

        if label_length == 0:
            label_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 rounded mt-1">'
                "📝 No Label"
                "</span>"
            )
        elif label_length < 3:
            label_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded mt-1">'
                "📝 Short"
                "</span>"
            )
        else:
            label_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded mt-1">'
                "📝 Good"
                "</span>"
            )

        return format_html(
            '<div class="space-y-1"><div>{}</div><div>{}</div></div>',
            status_badge,
            label_badge,
        )

    status_badge.short_description = _("Status")

    def usage_stats(self, obj):
        usage_count = getattr(obj, "usage_count", 0)

        if usage_count == 0:
            usage_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-base-50 dark:bg-base-800 text-base-700 dark:text-base-700 rounded">'
                "🚫 Unused"
                "</span>"
            )
        elif usage_count == 1:
            usage_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded">'
                "⚠️ Single"
                "</span>"
            )
        elif usage_count <= 10:
            usage_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded">'
                "📊 Low"
                "</span>"
            )
        elif usage_count <= 50:
            usage_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded">'
                "📈 Popular"
                "</span>"
            )
        else:
            usage_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-200 rounded">'
                "🌟 Trending"
                "</span>"
            )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{} uses</div>'
            '<div class="mt-1">{}</div>'
            "</div>",
            usage_count,
            usage_badge,
        )

    usage_stats.short_description = _("Usage")

    def content_distribution(self, obj):
        content_types_count = getattr(obj, "content_types_count", 0)

        if hasattr(obj, "taggeditem_set"):
            content_type_usage = {}
            for tagged_item in obj.taggeditem_set.all()[:10]:
                ct_name = tagged_item.content_type.model
                content_type_usage[ct_name] = (
                    content_type_usage.get(ct_name, 0) + 1
                )

            if content_type_usage:
                top_content_type = max(
                    content_type_usage, key=content_type_usage.get
                )
                top_count = content_type_usage[top_content_type]
            else:
                top_content_type = "None"
                top_count = 0
        else:
            top_content_type = "Unknown"
            top_count = 0

        if content_types_count == 0:
            diversity_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-base-50 dark:bg-base-800 text-base-700 dark:text-base-700 rounded">'
                "📊 No Data"
                "</span>"
            )
        elif content_types_count == 1:
            diversity_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded">'
                "📌 Focused"
                "</span>"
            )
        else:
            diversity_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded">'
                "🌐 Diverse"
                "</span>"
            )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{} types</div>'
            '<div class="text-base-600 dark:text-base-400">{}: {}</div>'
            '<div class="mt-1">{}</div>'
            "</div>",
            content_types_count,
            top_content_type.title()
            if top_content_type != "None"
            else "No usage",
            top_count if top_count > 0 else "",
            diversity_badge,
        )

    content_distribution.short_description = _("Content Types")

    def sort_display(self, obj):
        sort_order = obj.sort_order

        if sort_order is None:
            badge_class = (
                "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200"
            )
            icon = "❌"
            label = "No Order"
        elif sort_order == 0:
            badge_class = (
                "bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200"
            )
            icon = "🥇"
            label = "First"
        else:
            badge_class = "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200"
            icon = "📋"
            label = f"#{sort_order}"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} rounded gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>"
            "</div>",
            sort_order if sort_order is not None else "None",
            badge_class,
            icon,
            label,
        )

    sort_display.short_description = _("Sort Order")

    def created_display(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-600 dark:text-base-400">{}</div>'
            "</div>",
            obj.created_at.strftime("%Y-%m-%d"),
            obj.created_at.strftime("%H:%M"),
        )

    created_display.short_description = _("Created")

    def tag_analytics(self, obj):
        label = obj.safe_translation_getter("label", any_language=True) or ""
        label_length = len(label)
        usage_count = TaggedItem.objects.filter(tag=obj).count()

        words = label.split() if label else []
        word_count = len(words)

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Label Length:</strong></div><div>{} chars</div>"
            "<div><strong>Word Count:</strong></div><div>{} words</div>"
            "<div><strong>Total Usage:</strong></div><div>{} items</div>"
            "<div><strong>Active Status:</strong></div><div>{}</div>"
            "<div><strong>Has Label:</strong></div><div>{}</div>"
            "<div><strong>Sort Position:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            label_length,
            word_count,
            usage_count,
            "Active" if obj.active else "Inactive",
            "Yes" if label else "No",
            obj.sort_order if obj.sort_order is not None else "Not Set",
        )

    tag_analytics.short_description = _("Tag Analytics")

    def usage_analytics(self, obj):
        tagged_items = TaggedItem.objects.filter(tag=obj)
        total_usage = tagged_items.count()

        content_types = (
            tagged_items.values("content_type__model")
            .annotate(count=models.Count("id"))
            .order_by("-count")[:3]
        )

        thirty_days_ago = timezone.now() - timedelta(days=30)
        recent_usage = tagged_items.filter(
            created_at__gte=thirty_days_ago
        ).count()

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Total Usage:</strong></div><div>{}</div>"
            "<div><strong>Recent Usage:</strong></div><div>{} (30d)</div>"
            "<div><strong>Content Types:</strong></div><div>{}</div>"
            "<div><strong>Usage Rate:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            total_usage,
            recent_usage,
            len(content_types),
            "High"
            if total_usage > 20
            else "Medium"
            if total_usage > 5
            else "Low",
        )

    usage_analytics.short_description = _("Usage Analytics")

    def content_analytics(self, obj):
        tagged_items = TaggedItem.objects.filter(tag=obj)

        content_type_stats = {}
        for item in tagged_items.select_related("content_type")[:50]:
            ct_name = item.content_type.model
            content_type_stats[ct_name] = content_type_stats.get(ct_name, 0) + 1

        diversity_score = len(content_type_stats)
        most_used_type = (
            max(content_type_stats, key=content_type_stats.get)
            if content_type_stats
            else "None"
        )
        most_used_count = (
            content_type_stats.get(most_used_type, 0)
            if most_used_type != "None"
            else 0
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Diversity Score:</strong></div><div>{} types</div>"
            "<div><strong>Most Used Type:</strong></div><div>{}</div>"
            "<div><strong>Type Usage:</strong></div><div>{} items</div>"
            "<div><strong>Specialization:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            diversity_score,
            most_used_type.title() if most_used_type != "None" else "No data",
            most_used_count,
            "Specialized"
            if diversity_score == 1
            else "General"
            if diversity_score > 3
            else "Mixed",
        )

    content_analytics.short_description = _("Content Analytics")

    @action(
        description=_("Activate selected tags"),
        variant=ActionVariant.SUCCESS,
        icon="check_circle",
    )
    def activate_tags(self, request, queryset):
        updated = queryset.update(active=True)
        self.message_user(
            request,
            ngettext(
                _("%(count)d tag was activated."),
                _("%(count)d tags were activated."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    @action(
        description=_("Deactivate selected tags"),
        variant=ActionVariant.WARNING,
        icon="cancel",
    )
    def deactivate_tags(self, request, queryset):
        updated = queryset.update(active=False)
        self.message_user(
            request,
            ngettext(
                _("%(count)d tag was deactivated."),
                _("%(count)d tags were deactivated."),
                updated,
            )
            % {"count": updated},
            messages.WARNING,
        )

    @action(
        description=_("Update sort order"),
        variant=ActionVariant.INFO,
        icon="sort",
    )
    def update_sort_order(self, request, queryset):
        tags_with_usage = queryset.annotate(
            usage_count=models.Count("taggeditem")
        ).order_by("-usage_count", "translations__label")

        updated = 0
        for index, tag in enumerate(tags_with_usage):
            tag.sort_order = index
            tag.save(update_fields=["sort_order"])
            updated += 1

        self.message_user(
            request,
            _("Updated sort order for %(count)d tags.") % {"count": updated},
            messages.SUCCESS,
        )

    @action(
        description=_("Analyze tag usage"),
        variant=ActionVariant.PRIMARY,
        icon="analytics",
    )
    def analyze_usage(self, request, queryset):
        total_tags = queryset.count()
        active_tags = queryset.filter(active=True).count()
        used_tags = queryset.filter(taggeditem__isnull=False).distinct().count()

        self.message_user(
            request,
            _(
                "Analysis complete: %(total)d total tags, %(active)d active, %(used)d in use."
            )
            % {"total": total_tags, "active": active_tags, "used": used_tags},
            messages.INFO,
        )


@admin.register(TaggedItem)
class TaggedItemAdmin(ModelAdmin):
    list_fullwidth = True
    list_filter_submit = True

    list_display = [
        "tagged_item_info",
        "tag_display",
        "content_object_display",
        "content_type_badge",
        "created_display",
    ]
    list_filter = [
        ("tag", admin.RelatedOnlyFieldListFilter),
        ("content_type", admin.RelatedOnlyFieldListFilter),
        ("created_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "tag__translations__label",
        "object_id",
    ]
    readonly_fields = (
        "uuid",
        "created_at",
        "updated_at",
        "content_object",
    )
    list_select_related = ["tag", "content_type"]
    list_per_page = 100
    ordering = ["-created_at"]

    fieldsets = (
        (
            _("Tagged Item Information"),
            {
                "fields": (
                    "tag",
                    "content_type",
                    "object_id",
                    "content_object",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .select_related("tag", "content_type")
            .prefetch_related("tag__translations")
        )

    def tagged_item_info(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">Item #{}</div>'
            '<div class="text-base-600 dark:text-base-400">Object ID: {}</div>'
            '<div class="text-xs text-base-500 dark:text-base-400">UUID: {}</div>'
            "</div>",
            obj.id,
            obj.object_id,
            str(obj.uuid)[:8],
        )

    tagged_item_info.short_description = _("Tagged Item")

    def tag_display(self, obj):
        tag = obj.tag
        label = (
            tag.safe_translation_getter("label", any_language=True)
            or "Unnamed Tag"
        )

        if tag.active:
            status_color = "text-green-600 dark:text-green-400"
            status_icon = "✅"
        else:
            status_color = "text-red-600 dark:text-red-400"
            status_icon = "❌"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="flex items-center gap-1 {}">'
            "<span>{}</span>"
            "<span>Tag #{}</span>"
            "</div>"
            "</div>",
            label,
            status_color,
            status_icon,
            tag.id,
        )

    tag_display.short_description = _("Tag")

    def content_object_display(self, obj):
        try:
            content_obj = obj.content_object
            if content_obj:
                obj_str = str(content_obj)[:40]
                if len(str(content_obj)) > 40:
                    obj_str += "..."

                return format_html(
                    '<div class="text-sm">'
                    '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
                    '<div class="text-base-600 dark:text-base-400">ID: {}</div>'
                    "</div>",
                    obj_str,
                    obj.object_id,
                )
            else:
                return format_html(
                    '<div class="text-sm text-red-600 dark:text-red-400">Object not found</div>'
                )
        except Exception:
            return format_html(
                '<div class="text-sm text-red-600 dark:text-red-400">Error loading object</div>'
            )

    content_object_display.short_description = _("Content Object")

    def content_type_badge(self, obj):
        content_type = obj.content_type
        model_name = content_type.model

        type_configs = {
            "product": {
                "bg": "bg-green-50 dark:bg-green-900",
                "text": "text-green-700 dark:text-green-200",
                "icon": "🏷️",
            },
            "article": {
                "bg": "bg-blue-50 dark:bg-blue-900",
                "text": "text-blue-700 dark:text-blue-200",
                "icon": "📄",
            },
            "user": {
                "bg": "bg-purple-50 dark:bg-purple-900",
                "text": "text-purple-700 dark:text-purple-200",
                "icon": "👤",
            },
            "category": {
                "bg": "bg-orange-50 dark:bg-orange-900",
                "text": "text-orange-700 dark:text-orange-200",
                "icon": "📁",
            },
        }

        config = type_configs.get(
            model_name,
            {
                "bg": "bg-base-50 dark:bg-base-800",
                "text": "text-base-700 dark:text-base-700",
                "icon": "📦",
            },
        )

        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} {} rounded border gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>",
            config["bg"],
            config["text"],
            config["icon"],
            model_name.title(),
        )

    content_type_badge.short_description = _("Content Type")

    def created_display(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-600 dark:text-base-400">{}</div>'
            "</div>",
            obj.created_at.strftime("%Y-%m-%d"),
            obj.created_at.strftime("%H:%M"),
        )

    created_display.short_description = _("Created")



from __future__ import annotations

from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

from notification.models.notification import Notification
from notification.serializers.notification import NotificationSerializer


@extend_schema(
    operation_id="getNotificationsByIds",
    summary=_("Returns the notifications for a list of ids."),
    description=_("Returns the notifications for a list of ids."),
    tags=["Notifications"],
    request=None,
    responses=NotificationSerializer(many=True),
    methods=["POST"],
)
@api_view(["POST"])
def notifications_by_ids(request):
    notification_ids = request.data.get("ids", [])
    if not notification_ids:
        return Response(
            {"error": _("No notification ids provided.")},
            status=status.HTTP_400_BAD_REQUEST,
        )

    notifications = Notification.objects.filter(id__in=notification_ids)
    if not notifications:
        return Response(
            {"error": _("No notifications found for the provided ids.")},
            status=status.HTTP_404_NOT_FOUND,
        )

    serializer = NotificationSerializer(notifications, many=True)
    return Response(serializer.data)



from django.test import TestCase

from blog.factories.author import BlogAuthorFactory
from blog.factories.comment import BlogCommentFactory
from blog.factories.post import BlogPostFactory
from blog.factories.tag import BlogTagFactory
from blog.managers.post import BlogPostQuerySet
from blog.models.post import BlogPost
from user.factories.account import UserAccountFactory


class TestBlogPostManager(TestCase):
    def setUp(self):
        self.author1 = BlogAuthorFactory(user=UserAccountFactory())
        self.author2 = BlogAuthorFactory(user=UserAccountFactory())

        self.post1 = BlogPostFactory(author=self.author1)
        self.post2 = BlogPostFactory(author=self.author2)
        self.post3 = BlogPostFactory(author=self.author1)

        self.comment1 = BlogCommentFactory(
            post=self.post1, user=UserAccountFactory()
        )
        self.comment2 = BlogCommentFactory(
            post=self.post1, user=UserAccountFactory()
        )
        self.comment3 = BlogCommentFactory(
            post=self.post2, user=UserAccountFactory()
        )

        self.tag1 = BlogTagFactory(active=True)
        self.tag2 = BlogTagFactory(active=True)
        self.tag3 = BlogTagFactory(active=False)

        self.post1.tags.add(self.tag1, self.tag2, self.tag3)
        self.post2.tags.add(self.tag1)

    def test_with_likes_count_manager_method(self):
        posts = BlogPost.objects.with_likes_count()

        assert posts.count() == 3

        for post in posts:
            assert hasattr(post, "likes_count_field")
            assert isinstance(post.likes_count_field, int)

    def test_with_comments_count_manager_method(self):
        posts = BlogPost.objects.with_comments_count()

        assert posts.count() == 3

        post1_annotated = posts.get(id=self.post1.id)
        post2_annotated = posts.get(id=self.post2.id)
        post3_annotated = posts.get(id=self.post3.id)

        assert post1_annotated.comments_count_field == 2
        assert post2_annotated.comments_count_field == 1
        assert post3_annotated.comments_count_field == 0

    def test_with_tags_count_manager_method(self):
        posts = BlogPost.objects.with_tags_count()

        assert posts.count() == 3

        post1_annotated = posts.get(id=self.post1.id)
        post2_annotated = posts.get(id=self.post2.id)
        post3_annotated = posts.get(id=self.post3.id)

        assert post1_annotated.tags_count_field == 2
        assert post2_annotated.tags_count_field == 1
        assert post3_annotated.tags_count_field == 0

    def test_with_all_annotations_manager_method(self):
        posts = BlogPost.objects.with_all_annotations()

        assert posts.count() == 3

        for post in posts:
            assert hasattr(post, "likes_count_field")
            assert hasattr(post, "comments_count_field")
            assert hasattr(post, "tags_count_field")
            assert isinstance(post.likes_count_field, int)
            assert isinstance(post.comments_count_field, int)
            assert isinstance(post.tags_count_field, int)

    def test_manager_returns_correct_queryset_type(self):
        queryset = BlogPost.objects.with_likes_count()

        assert isinstance(queryset, BlogPostQuerySet)


class TestBlogPostQuerySet(TestCase):
    def setUp(self):
        self.author1 = BlogAuthorFactory(user=UserAccountFactory())
        self.author2 = BlogAuthorFactory(user=UserAccountFactory())

        self.post1 = BlogPostFactory(author=self.author1)
        self.post2 = BlogPostFactory(author=self.author2)
        self.post3 = BlogPostFactory(author=self.author1)

        self.user1 = UserAccountFactory()
        self.user2 = UserAccountFactory()
        self.user3 = UserAccountFactory()

        self.post1.likes.add(self.user1, self.user2)
        self.post2.likes.add(self.user1)

        BlogCommentFactory(post=self.post1, user=self.user1)
        BlogCommentFactory(post=self.post1, user=self.user2)
        BlogCommentFactory(post=self.post1, user=self.user3)
        BlogCommentFactory(post=self.post2, user=self.user1)

        self.active_tag1 = BlogTagFactory(active=True)
        self.active_tag2 = BlogTagFactory(active=True)
        self.inactive_tag = BlogTagFactory(active=False)

        self.post1.tags.add(
            self.active_tag1, self.active_tag2, self.inactive_tag
        )
        self.post2.tags.add(self.active_tag1)
        self.post3.tags.add(self.inactive_tag)

    def test_with_likes_count_queryset_method(self):
        queryset = BlogPost.objects.filter().with_likes_count()

        assert queryset.count() == 3

        post1_annotated = queryset.get(id=self.post1.id)
        post2_annotated = queryset.get(id=self.post2.id)
        post3_annotated = queryset.get(id=self.post3.id)

        assert post1_annotated.likes_count_field == 2
        assert post2_annotated.likes_count_field == 1
        assert post3_annotated.likes_count_field == 0

    def test_with_comments_count_queryset_method(self):
        queryset = BlogPost.objects.filter().with_comments_count()

        assert queryset.count() == 3

        post1_annotated = queryset.get(id=self.post1.id)
        post2_annotated = queryset.get(id=self.post2.id)
        post3_annotated = queryset.get(id=self.post3.id)

        assert post1_annotated.comments_count_field == 3
        assert post2_annotated.comments_count_field == 1
        assert post3_annotated.comments_count_field == 0

    def test_with_tags_count_queryset_method(self):
        queryset = BlogPost.objects.filter().with_tags_count()

        assert queryset.count() == 3

        post1_annotated = queryset.get(id=self.post1.id)
        post2_annotated = queryset.get(id=self.post2.id)
        post3_annotated = queryset.get(id=self.post3.id)

        assert post1_annotated.tags_count_field == 2
        assert post2_annotated.tags_count_field == 1
        assert post3_annotated.tags_count_field == 0

    def test_with_all_annotations_queryset_method(self):
        queryset = BlogPost.objects.filter().with_all_annotations()

        assert queryset.count() == 3

        post1_annotated = queryset.get(id=self.post1.id)

        assert post1_annotated.likes_count_field == 2
        assert post1_annotated.comments_count_field == 3
        assert post1_annotated.tags_count_field == 2

    def test_queryset_chaining(self):
        queryset = (
            BlogPost.objects.filter(author=self.author1)
            .with_likes_count()
            .with_comments_count()
            .with_tags_count()
        )

        assert queryset.count() == 2

        for post in queryset:
            assert hasattr(post, "likes_count_field")
            assert hasattr(post, "comments_count_field")
            assert hasattr(post, "tags_count_field")

    def test_queryset_with_ordering(self):
        queryset = BlogPost.objects.with_likes_count().order_by(
            "-likes_count_field"
        )

        posts = list(queryset)

        assert posts[0].id == self.post1.id
        assert posts[1].id == self.post2.id
        assert posts[2].id == self.post3.id

    def test_queryset_with_filtering(self):
        queryset = BlogPost.objects.with_comments_count().filter(
            comments_count_field__gt=0
        )

        assert queryset.count() == 2
        assert self.post1 in queryset
        assert self.post2 in queryset
        assert self.post3 not in queryset

    def test_distinct_behavior_with_annotations(self):
        self.post1.likes.add(UserAccountFactory())
        BlogCommentFactory(post=self.post1, user=UserAccountFactory())

        queryset = BlogPost.objects.with_all_annotations()

        post_ids = list(queryset.values_list("id", flat=True))
        assert len(post_ids) == len(set(post_ids))

    def test_queryset_returns_correct_type(self):
        queryset = BlogPost.objects.filter().with_likes_count()

        assert isinstance(queryset, BlogPostQuerySet)

        chained = queryset.with_comments_count()
        assert isinstance(chained, BlogPostQuerySet)


class TestBlogPostManagerEdgeCases(TestCase):
    def test_empty_queryset_behavior(self):
        assert BlogPost.objects.with_likes_count().count() == 0
        assert BlogPost.objects.with_comments_count().count() == 0
        assert BlogPost.objects.with_tags_count().count() == 0
        assert BlogPost.objects.with_all_annotations().count() == 0

    def test_posts_without_relations(self):
        author = BlogAuthorFactory(user=UserAccountFactory())
        post = BlogPostFactory(author=author)

        annotated_posts = BlogPost.objects.with_all_annotations()
        annotated_post = annotated_posts.get(id=post.id)

        assert annotated_post.likes_count_field == 0
        assert annotated_post.comments_count_field == 0
        assert annotated_post.tags_count_field == 0

    def test_posts_with_inactive_tags_only(self):
        author = BlogAuthorFactory(user=UserAccountFactory())
        post = BlogPostFactory(author=author)

        inactive_tag1 = BlogTagFactory(active=False)
        inactive_tag2 = BlogTagFactory(active=False)
        post.tags.add(inactive_tag1, inactive_tag2)

        annotated_posts = BlogPost.objects.with_tags_count()
        annotated_post = annotated_posts.get(id=post.id)

        assert annotated_post.tags_count_field == 0

    def test_large_dataset_performance(self):
        author = BlogAuthorFactory(user=UserAccountFactory())

        posts = []
        for i in range(20):
            post = BlogPostFactory(author=author)
            posts.append(post)

            for j in range(i % 5):
                user = UserAccountFactory()
                post.likes.add(user)
                BlogCommentFactory(post=post, user=user)

                if j < 3:
                    tag = BlogTagFactory(active=True)
                    post.tags.add(tag)

        annotated_posts = BlogPost.objects.with_all_annotations()
        assert annotated_posts.count() == 20

        for post in annotated_posts:
            assert hasattr(post, "likes_count_field")
            assert hasattr(post, "comments_count_field")
            assert hasattr(post, "tags_count_field")

    def test_annotation_with_deleted_relations(self):
        author = BlogAuthorFactory(user=UserAccountFactory())
        post = BlogPostFactory(author=author)

        user = UserAccountFactory()
        post.likes.add(user)
        comment = BlogCommentFactory(post=post, user=user)
        tag = BlogTagFactory(active=True)
        post.tags.add(tag)

        annotated_post = BlogPost.objects.with_all_annotations().get(id=post.id)
        assert annotated_post.likes_count_field == 1
        assert annotated_post.comments_count_field == 1
        assert annotated_post.tags_count_field == 1

        comment.delete()
        tag.delete()
        post.likes.remove(user)

        annotated_post = BlogPost.objects.with_all_annotations().get(id=post.id)
        assert annotated_post.likes_count_field == 0
        assert annotated_post.comments_count_field == 0
        assert annotated_post.tags_count_field == 0

    def test_manager_get_queryset_method(self):
        manager = BlogPost.objects
        queryset = manager.get_queryset()

        assert isinstance(queryset, BlogPostQuerySet)

        assert hasattr(queryset, "with_likes_count")
        assert hasattr(queryset, "with_comments_count")
        assert hasattr(queryset, "with_tags_count")
        assert hasattr(queryset, "with_all_annotations")



from __future__ import annotations

from typing import TYPE_CHECKING

from allauth.socialaccount.signals import (
    pre_social_login,
    social_account_added,
    social_account_removed,
    social_account_updated,
)
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.crypto import get_random_string

from user.models.subscription import SubscriptionTopic, UserSubscription
from user.utils.subscription import send_subscription_confirmation

if TYPE_CHECKING:  # pragma: no cover
    from allauth.socialaccount.models import SocialAccount, SocialLogin

User = get_user_model()


@receiver(pre_social_login)
def pre_social_login_signal(
    sender, request, sociallogin: SocialLogin, **kwargs
):
    pass


@receiver(social_account_added)
def social_account_added_signal(sender, **kwargs):
    pass


@receiver(social_account_updated)
def social_account_updated_signal(
    sender, request, sociallogin: SocialLogin, **kwargs
):
    pass


@receiver(social_account_removed)
def social_account_removed_signal(
    sender, request, socialaccount: SocialAccount, **kwargs
):
    pass


@receiver(post_save, sender=User)
def create_default_subscriptions(sender, instance, created, **kwargs):
    if created:
        default_topics = SubscriptionTopic.objects.filter(
            is_active=True, is_default=True
        )

        for topic in default_topics:
            subscription = UserSubscription.objects.create(
                user=instance,
                topic=topic,
                status=(
                    UserSubscription.SubscriptionStatus.PENDING
                    if topic.requires_confirmation
                    else UserSubscription.SubscriptionStatus.ACTIVE
                ),
            )

            if topic.requires_confirmation:
                subscription.confirmation_token = get_random_string(64)
                subscription.save()
                send_subscription_confirmation(subscription, instance)



import importlib

import factory
from django.apps import apps
from django.conf import settings
from faker import Faker

from core.factories import CustomDjangoModelFactory
from region.models import Region

fake = Faker()

available_languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


def get_or_create_country():
    if apps.get_model("country", "Country").objects.exists():
        return (
            apps.get_model("country", "Country").objects.order_by("?").first()
        )
    else:
        country_factory_module = importlib.import_module("country.factories")
        country_factory_class = country_factory_module.CountryFactory
        return country_factory_class.create()


class RegionTranslationFactory(factory.django.DjangoModelFactory):
    language_code = factory.Iterator(available_languages)
    name = factory.Faker("city")
    master = factory.SubFactory("region.factories.RegionFactory")

    class Meta:
        model = apps.get_model("region", "RegionTranslation")
        django_get_or_create = ("language_code", "master")


class RegionFactory(CustomDjangoModelFactory):
    unique_model_fields = [
        ("alpha", lambda: fake.bothify(text="???-######")),
    ]

    country = factory.LazyFunction(get_or_create_country)

    class Meta:
        model = Region
        django_get_or_create = ("alpha",)
        skip_postgeneration_save = True

    @factory.post_generation
    def translations(self, create, extracted, **kwargs):
        if not create:
            return

        translations = extracted or [
            RegionTranslationFactory(language_code=lang, master=self)
            for lang in [
                lang["code"]
                for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
            ]
        ]

        for translation in translations:
            translation.save()



from __future__ import annotations

from django.conf import settings
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema_view
from rest_framework.filters import SearchFilter

from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from product.models.image import ProductImage
from product.serializers.image import (
    ProductImageDetailSerializer,
    ProductImageSerializer,
    ProductImageWriteSerializer,
)


@extend_schema_view(
    **create_schema_view_config(
        model_class=ProductImage,
        display_config={
            "tag": "Product Images",
        },
        serializers={
            "list_serializer": ProductImageSerializer,
            "detail_serializer": ProductImageDetailSerializer,
            "write_serializer": ProductImageWriteSerializer,
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class ProductImageViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = ProductImage.objects.all()
    serializers = {
        "default": ProductImageDetailSerializer,
        "list": ProductImageSerializer,
        "retrieve": ProductImageDetailSerializer,
        "create": ProductImageWriteSerializer,
        "update": ProductImageWriteSerializer,
        "partial_update": ProductImageWriteSerializer,
    }
    response_serializers = {
        "create": ProductImageDetailSerializer,
        "update": ProductImageDetailSerializer,
        "partial_update": ProductImageDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_fields = [
        "id",
        "product",
        "is_main",
        "sort_order",
        "created_at",
        "updated_at",
    ]
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "is_main",
        "sort_order",
    ]
    ordering = ["-is_main", "sort_order", "-created_at"]

    def get_queryset(self):
        queryset = ProductImage.objects.all()

        if self.action == "list":
            return queryset.optimized_for_list().ordered_by_position()
        elif self.action in ["retrieve", "update", "partial_update", "destroy"]:
            return queryset.with_product_data()

        return queryset



from __future__ import annotations

from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import (
    extend_schema,
    extend_schema_view,
)
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.generics import get_object_or_404
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from authentication.serializers import AuthenticationSerializer
from blog.serializers.comment import BlogCommentSerializer
from blog.serializers.post import BlogPostSerializer
from core.api.permissions import IsSelfOrAdmin
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from notification.serializers.user import NotificationUserSerializer
from order.serializers.order import OrderSerializer
from product.serializers.favourite import ProductFavouriteSerializer
from product.serializers.review import ProductReviewSerializer
from user.filters.account import UserAccountFilter
from user.models.subscription import SubscriptionTopic, UserSubscription
from user.serializers.account import (
    UsernameUpdateResponseSerializer,
    UsernameUpdateSerializer,
    UserSubscriptionSummaryResponseSerializer,
)
from user.serializers.address import UserAddressSerializer
from user.serializers.subscription import UserSubscriptionSerializer
from user.utils.subscription import get_user_subscription_summary

User = get_user_model()


@extend_schema_view(
    **create_schema_view_config(
        model_class=User,
        display_config={
            "tag": "User Accounts",
        },
        serializers={
            "list_serializer": AuthenticationSerializer,
            "detail_serializer": AuthenticationSerializer,
            "write_serializer": AuthenticationSerializer,
        },
        error_serializer=ErrorResponseSerializer,
    ),
    favourite_products=extend_schema(
        operation_id="getUserAccountFavouriteProducts",
        summary=_("Get user's favourite products"),
        description=_("Get all favourite products for a specific user."),
        tags=["User Accounts"],
        responses={
            200: ProductFavouriteSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    orders=extend_schema(
        operation_id="getUserAccountOrders",
        summary=_("Get user's orders"),
        description=_("Get all orders for a specific user."),
        tags=["User Accounts"],
        responses={
            200: OrderSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    product_reviews=extend_schema(
        operation_id="getUserAccountProductReviews",
        summary=_("Get user's product reviews"),
        description=_("Get all product reviews written by a specific user."),
        tags=["User Accounts"],
        responses={
            200: ProductReviewSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    addresses=extend_schema(
        operation_id="getUserAccountAddresses",
        summary=_("Get user's addresses"),
        description=_("Get all addresses for a specific user."),
        tags=["User Accounts"],
        responses={
            200: UserAddressSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    blog_post_comments=extend_schema(
        operation_id="getUserAccountBlogPostComments",
        summary=_("Get user's blog comments"),
        description=_("Get all blog post comments written by a specific user."),
        tags=["User Accounts"],
        responses={
            200: BlogCommentSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    liked_blog_posts=extend_schema(
        operation_id="getUserAccountLikedBlogPosts",
        summary=_("Get user's liked blog posts"),
        description=_("Get all blog posts liked by a specific user."),
        tags=["User Accounts"],
        responses={
            200: BlogPostSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    notifications=extend_schema(
        operation_id="getUserAccountNotifications",
        summary=_("Get user's notifications"),
        description=_("Get all notifications for a specific user."),
        tags=["User Accounts"],
        responses={
            200: NotificationUserSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    subscriptions=extend_schema(
        operation_id="getUserAccountSubscriptions",
        summary=_("Get user's subscriptions"),
        description=_("Get all subscriptions for a specific user."),
        tags=["User Accounts"],
        responses={
            200: UserSubscriptionSerializer(many=True),
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    change_username=extend_schema(
        operation_id="changeUserAccountUsername",
        summary=_("Change username"),
        description=_("Change the username for a specific user."),
        tags=["User Accounts"],
        request=UsernameUpdateSerializer,
        responses={
            200: UsernameUpdateResponseSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    subscription_summary=extend_schema(
        operation_id="getUserAccountSubscriptionSummary",
        summary=_("Get user's subscription summary"),
        description=_("Get a summary of subscriptions for a specific user."),
        tags=["User Accounts"],
        responses={
            200: UserSubscriptionSummaryResponseSerializer,
            401: ErrorResponseSerializer,
            403: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
)
class UserAccountViewSet(MultiSerializerMixin, BaseModelViewSet):
    permission_classes = [IsAuthenticated, IsSelfOrAdmin]
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = UserAccountFilter
    ordering_fields = ["id", "email", "username", "created_at", "updated_at"]
    ordering = ["-created_at"]
    search_fields = ["id", "email", "username", "first_name", "last_name"]

    serializers = {
        "default": AuthenticationSerializer,
        "create": AuthenticationSerializer,
        "favourite_products": ProductFavouriteSerializer,
        "orders": OrderSerializer,
        "product_reviews": ProductReviewSerializer,
        "addresses": UserAddressSerializer,
        "blog_post_comments": BlogCommentSerializer,
        "liked_blog_posts": BlogPostSerializer,
        "notifications": NotificationUserSerializer,
        "subscriptions": UserSubscriptionSerializer,
        "change_username": UsernameUpdateSerializer,
    }

    def get_queryset(self):
        if getattr(self, "swagger_fake_view", False):
            return User.objects.none()

        match self.action:
            case "favourite_products":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).favourite_products.all()
            case "orders":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).orders.all()
            case "product_reviews":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).product_reviews.all()
            case "addresses":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).addresses.all()
            case "blog_post_comments":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).blog_comments.all()
            case "liked_blog_posts":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).liked_blog_posts.all()
            case "notifications":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).notification.all()
            case "subscriptions":
                queryset = get_object_or_404(
                    User, id=self.kwargs["pk"]
                ).subscriptions.select_related("topic")
            case _:
                queryset = (
                    User.objects.all()
                    if self.request.user.is_staff
                    else User.objects.filter(id=self.request.user.id)
                )

        return queryset

    def get_object(self):
        obj = super().get_object()
        self.check_object_permissions(self.request, obj)
        return obj

    @action(detail=True, methods=["GET"])
    def favourite_products(self, request, pk=None):
        self.filterset_fields = ["id"]
        self.ordering_fields = [
            "id",
            "product_id",
            "created_at",
            "updated_at",
        ]
        self.ordering = ["-updated_at"]
        self.search_fields = [
            "id",
            "product_id",
        ]
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def orders(self, request, pk=None):
        self.ordering_fields = ["created_at", "updated_at", "status"]
        self.filterset_fields = ["status"]
        self.ordering = ["-created_at"]
        self.search_fields = []
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def product_reviews(self, request, pk=None):
        self.filterset_fields = ["id", "product_id", "status"]
        self.ordering_fields = [
            "id",
            "product_id",
            "created_at",
            "updated_at",
        ]
        self.ordering = ["-created_at"]
        self.search_fields = [
            "id",
            "product_id",
        ]
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def addresses(self, request, pk=None):
        self.filterset_fields = [
            "id",
            "country",
            "city",
            "street",
            "zipcode",
            "floor",
            "location_type",
            "is_main",
        ]
        self.ordering_fields = [
            "id",
            "country",
            "zipcode",
            "floor",
            "location_type",
            "is_main",
            "created_at",
            "updated_at",
        ]
        self.ordering = ["-is_main", "-created_at"]
        self.search_fields = ["id", "country", "city", "street", "zipcode"]
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def blog_post_comments(self, request, pk=None):
        self.filterset_fields = ["id", "post", "parent", "approved"]
        self.ordering_fields = ["id", "post", "created_at"]
        self.ordering = ["-created_at"]
        self.search_fields = ["id", "post"]
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def liked_blog_posts(self, request, pk=None):
        self.filterset_fields = ["id", "tags", "slug", "author"]
        self.ordering_fields = [
            "id",
            "title",
            "created_at",
            "updated_at",
            "published_at",
        ]
        self.ordering = ["-created_at"]
        self.search_fields = ["id", "title", "subtitle", "body"]
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET"])
    def notifications(self, request, pk=None):
        self.filterset_fields = ["seen", "notification__kind"]
        self.ordering_fields = ["created_at", "seen_at"]
        self.ordering = ["-created_at"]

        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(queryset, request)

    @action(detail=True, methods=["GET", "POST"])
    def subscriptions(self, request, pk=None):
        user = self.get_object()

        if request.method == "GET":
            self.filterset_fields = ["topic", "status", "topic__category"]
            self.ordering_fields = ["subscribed_at", "created_at"]
            self.ordering = ["-subscribed_at"]
            self.search_fields = ["topic__name", "topic__description"]

            queryset = self.filter_queryset(self.get_queryset())
            return self.paginate_and_serialize(queryset, request)

        elif request.method == "POST":
            topic_id = request.data.get("topic_id")
            if not topic_id:
                return Response(
                    {"detail": _("Topic ID is required.")},
                    status=status.HTTP_400_BAD_REQUEST,
                )

            try:
                topic = SubscriptionTopic.objects.get(
                    id=topic_id, is_active=True
                )
            except SubscriptionTopic.DoesNotExist:
                return Response(
                    {"detail": _("Topic not found or not active.")},
                    status=status.HTTP_404_NOT_FOUND,
                )

            subscription, created = UserSubscription.objects.get_or_create(
                user=user,
                topic=topic,
                defaults={
                    "status": UserSubscription.SubscriptionStatus.PENDING
                    if topic.requires_confirmation
                    else UserSubscription.SubscriptionStatus.ACTIVE
                },
            )

            if (
                not created
                and subscription.status
                == UserSubscription.SubscriptionStatus.ACTIVE
            ):
                return Response(
                    {"detail": _("User is already subscribed to this topic.")},
                    status=status.HTTP_400_BAD_REQUEST,
                )

            serializer = UserSubscriptionSerializer(subscription)
            return Response(serializer.data, status=status.HTTP_201_CREATED)

    @action(detail=True, methods=["POST"])
    def change_username(self, request, pk=None):
        user = self.get_object()

        serializer = UsernameUpdateSerializer(data=request.data)
        if serializer.is_valid():
            new_username = serializer.validated_data["username"]

            if (
                User.objects.filter(username=new_username)
                .exclude(id=user.id)
                .exists()
            ):
                return Response(
                    {"detail": _("Username already exists.")},
                    status=status.HTTP_400_BAD_REQUEST,
                )

            user.username = new_username
            user.save()

            return Response(
                {"detail": _("Username updated successfully.")},
                status=status.HTTP_200_OK,
            )

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=["GET"])
    def subscription_summary(self, request, pk=None):
        user = self.get_object()
        summary = get_user_subscription_summary(user)
        return Response(summary)



from __future__ import annotations

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.response import Response

from blog.filters.author import BlogAuthorFilter
from blog.models.author import BlogAuthor
from blog.serializers.author import (
    BlogAuthorDetailSerializer,
    BlogAuthorSerializer,
    BlogAuthorWriteSerializer,
)
from blog.serializers.post import BlogPostSerializer
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods


@extend_schema_view(
    **create_schema_view_config(
        model_class=BlogAuthor,
        display_config={
            "tag": "Blog Authors",
        },
        serializers={
            "list_serializer": BlogAuthorSerializer,
            "detail_serializer": BlogAuthorDetailSerializer,
            "write_serializer": BlogAuthorWriteSerializer,
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class BlogAuthorViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = BlogAuthor.objects.all()
    serializers = {
        "default": BlogAuthorDetailSerializer,
        "list": BlogAuthorSerializer,
        "retrieve": BlogAuthorDetailSerializer,
        "create": BlogAuthorWriteSerializer,
        "update": BlogAuthorWriteSerializer,
        "partial_update": BlogAuthorWriteSerializer,
        "posts": BlogPostSerializer,
    }
    response_serializers = {
        "create": BlogAuthorDetailSerializer,
        "update": BlogAuthorDetailSerializer,
        "partial_update": BlogAuthorDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = BlogAuthorFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "user__first_name",
        "user__last_name",
        "user__date_joined",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "user__first_name",
        "user__last_name",
        "user__email",
        "website",
        "translations__bio",
    ]

    @extend_schema(
        operation_id="getBlogAuthorPosts",
        summary=_("Get author's blog posts"),
        description=_(
            "Retrieve all blog posts written by this author with proper pagination."
        ),
        tags=["Blog Authors"],
        responses={
            200: BlogPostSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def posts(self, request, pk=None, *args, **kwargs):
        author = self.get_object()
        queryset = author.blog_posts.all()

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = BlogPostSerializer(
                page, many=True, context=self.get_serializer_context()
            )
            return self.get_paginated_response(serializer.data)

        serializer = BlogPostSerializer(
            queryset, many=True, context=self.get_serializer_context()
        )
        return Response(serializer.data)



import django_filters
from django.db.models import Count
from django.utils.translation import gettext_lazy as _

from blog.models.category import BlogCategory


class BlogCategoryFilter(django_filters.FilterSet):
    parent = django_filters.NumberFilter(
        field_name="parent__id", help_text=_("Filter by parent category ID")
    )
    parent__isnull = django_filters.BooleanFilter(
        field_name="parent",
        lookup_expr="isnull",
        help_text=_("Filter root categories (true) or non-root (false)"),
    )
    level = django_filters.NumberFilter(
        help_text=_("Filter by tree level (0 for root categories)")
    )
    level__gte = django_filters.NumberFilter(
        field_name="level",
        lookup_expr="gte",
        help_text=_("Filter categories at or below this level"),
    )
    level__lte = django_filters.NumberFilter(
        field_name="level",
        lookup_expr="lte",
        help_text=_("Filter categories at or above this level"),
    )

    has_posts = django_filters.BooleanFilter(
        method="filter_has_posts",
        help_text=_(
            "Filter categories that have posts (true) or no posts (false)"
        ),
    )
    min_post_count = django_filters.NumberFilter(
        method="filter_min_post_count",
        help_text=_("Filter categories with at least this many posts"),
    )

    name = django_filters.CharFilter(
        field_name="translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by category name (case-insensitive)"),
    )
    description = django_filters.CharFilter(
        field_name="translations__description",
        lookup_expr="icontains",
        help_text=_("Filter by description content (case-insensitive)"),
    )

    class Meta:
        model = BlogCategory
        fields = {
            "id": ["exact", "in"],
            "slug": ["exact", "icontains"],
            "sort_order": ["exact", "gte", "lte"],
        }

    def filter_has_posts(self, queryset, name, value):
        if value is True:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__gt=0
            )
        elif value is False:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count=0
            )
        return queryset

    def filter_min_post_count(self, queryset, name, value):
        return queryset.annotate(post_count=Count("blog_posts")).filter(
            post_count__gte=value
        )



from django.contrib import admin, messages
from django.db import models
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from parler.admin import TranslatableAdmin
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
    RangeNumericFilter,
)
from unfold.decorators import action
from unfold.enums import ActionVariant

from country.models import Country


class CountryStatusFilter(DropdownFilter):
    title = _("Country Status")
    parameter_name = "country_status"

    def lookups(self, request, model_admin):
        return [
            ("active", _("Active (Has ISO Code)")),
            ("incomplete", _("Incomplete (Missing Data)")),
            ("with_phone", _("Has Phone Code")),
            ("with_flag", _("Has Flag Image")),
            ("complete", _("Complete Profile")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "active":
            return queryset.filter(iso_cc__isnull=False)
        elif self.value() == "incomplete":
            return queryset.filter(
                models.Q(iso_cc__isnull=True)
                | models.Q(phone_code__isnull=True)
                | models.Q(image_flag__isnull=True)
            )
        elif self.value() == "with_phone":
            return queryset.filter(phone_code__isnull=False)
        elif self.value() == "with_flag":
            return queryset.filter(image_flag__isnull=False)
        elif self.value() == "complete":
            return queryset.filter(
                iso_cc__isnull=False,
                phone_code__isnull=False,
                image_flag__isnull=False,
            )
        return queryset


@admin.register(Country)
class CountryAdmin(ModelAdmin, TranslatableAdmin):
    list_fullwidth = True
    list_filter_submit = True

    list_display = [
        "country_info",
        "flag_display",
        "codes_display",
        "contact_info",
        "completeness_badge",
        "created_display",
    ]
    list_filter = [
        CountryStatusFilter,
        ("iso_cc", RangeNumericFilter),
        ("phone_code", RangeNumericFilter),
        ("created_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "translations__name",
        "alpha_2",
        "alpha_3",
        "iso_cc",
        "phone_code",
    ]
    readonly_fields = (
        "uuid",
        "sort_order",
        "created_at",
        "updated_at",
    )
    list_per_page = 50
    ordering = ["sort_order", "alpha_2"]
    actions = [
        "update_sort_order",
    ]

    fieldsets = (
        (
            _("Country Information"),
            {
                "fields": ("name",),
                "classes": ("wide",),
            },
        ),
        (
            _("Country Codes"),
            {
                "fields": ("alpha_2", "alpha_3", "iso_cc"),
                "classes": ("wide",),
            },
        ),
        (
            _("Contact & Media"),
            {
                "fields": ("phone_code", "image_flag"),
                "classes": ("wide",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "sort_order", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def country_info(self, obj):
        name = (
            obj.safe_translation_getter("name", any_language=True)
            or "Unnamed Country"
        )

        has_iso = obj.iso_cc is not None
        status_icon = "✅" if has_iso else "⚠️"
        status_color = (
            "text-green-600 dark:text-green-400"
            if has_iso
            else "text-orange-600 dark:text-orange-400"
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100 flex items-center gap-2">'
            '<span class="{}">{}</span>'
            "<span>{}</span>"
            "</div>"
            '<div class="text-base-600 dark:text-base-400">{}</div>'
            '<div class="text-xs text-base-500 dark:text-base-400">Sort: {}</div>'
            "</div>",
            status_color,
            status_icon,
            name,
            obj.alpha_2,
            obj.sort_order or "No order",
        )

    country_info.short_description = _("Country")

    def flag_display(self, obj):
        if obj.image_flag:
            return format_html(
                '<div class="flex items-center justify-center">'
                '<img src="{}" style="width: 48px; height: 32px; object-fit: cover; border-radius: 4px; border: 1px solid #e5e7eb; box-shadow: 0 1px 3px rgba(0,0,0,0.1);" />'
                "</div>",
                obj.image_flag.url,
            )
        else:
            return format_html(
                '<div style="width: 48px; height: 32px; background: linear-gradient(45deg, #f3f4f6 25%, transparent 25%), linear-gradient(-45deg, #f3f4f6 25%, transparent 25%), linear-gradient(45deg, transparent 75%, #f3f4f6 75%), linear-gradient(-45deg, transparent 75%, #f3f4f6 75%); background-size: 8px 8px; background-position: 0 0, 0 4px, 4px -4px, -4px 0px; border-radius: 4px; border: 1px solid #e5e7eb; display: flex; align-items: center; justify-content: center; color: #9ca3af; font-size: 12px;">'
                "🏳️"
                "</div>"
            )

    flag_display.short_description = _("Flag")

    def codes_display(self, obj):
        alpha_2_badge = format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded border border-blue-200 dark:border-blue-700">'
            "{}"
            "</span>",
            obj.alpha_2,
        )

        alpha_3_badge = format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded border border-green-200 dark:border-green-700">'
            "{}"
            "</span>",
            obj.alpha_3,
        )

        iso_display = str(obj.iso_cc) if obj.iso_cc else "—"
        iso_color = (
            "text-base-900 dark:text-base-100"
            if obj.iso_cc
            else "text-base-400 dark:text-base-500"
        )

        return format_html(
            '<div class="text-sm space-y-1">'
            '<div class="flex gap-1">{} {}</div>'
            '<div class="{}">ISO: {}</div>'
            "</div>",
            alpha_2_badge,
            alpha_3_badge,
            iso_color,
            iso_display,
        )

    codes_display.short_description = _("Codes")

    def contact_info(self, obj):
        if obj.phone_code:
            code_str = str(obj.phone_code)
            if len(code_str) == 1:
                badge_class = "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 border-green-200 dark:border-green-700"
            elif len(code_str) == 2:
                badge_class = "bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 border-blue-200 dark:border-blue-700"
            elif len(code_str) == 3:
                badge_class = "bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-200 border-orange-200 dark:border-orange-700"
            else:
                badge_class = "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 border-red-200 dark:border-red-700"

            phone_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} rounded border">'
                "📞 +{}"
                "</span>",
                badge_class,
                obj.phone_code,
            )
        else:
            phone_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-gray-50 dark:bg-gray-800 text-base-500 dark:text-base-400 rounded border border-gray-200 dark:border-gray-700">'
                "📞 No Code"
                "</span>"
            )

        return format_html(
            '<div class="text-sm"><div>{}</div></div>',
            phone_badge,
        )

    contact_info.short_description = _("Contact")

    def completeness_badge(self, obj):
        total_fields = 4
        completed_fields = 0

        if obj.safe_translation_getter("name", any_language=True):
            completed_fields += 1
        if obj.iso_cc is not None:
            completed_fields += 1
        if obj.phone_code is not None:
            completed_fields += 1
        if obj.image_flag:
            completed_fields += 1

        percentage = (completed_fields / total_fields) * 100

        if percentage == 100:
            badge_class = "bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 border-green-200 dark:border-green-700"
            icon = "✅"
            label = "Complete"
        elif percentage >= 75:
            badge_class = "bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 border-blue-200 dark:border-blue-700"
            icon = "🔷"
            label = "Good"
        elif percentage >= 50:
            badge_class = "bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 border-yellow-200 dark:border-yellow-700"
            icon = "⚠️"
            label = "Partial"
        else:
            badge_class = "bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 border-red-200 dark:border-red-700"
            icon = "❌"
            label = "Incomplete"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}%</div>'
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium {} rounded border gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>"
            "</div>",
            int(percentage),
            badge_class,
            icon,
            label,
        )

    completeness_badge.short_description = _("Completeness")

    def created_display(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-600 dark:text-base-400">{}</div>'
            "</div>",
            obj.created_at.strftime("%Y-%m-%d"),
            obj.created_at.strftime("%H:%M"),
        )

    created_display.short_description = _("Created")

    @action(
        description=_("Update sort order"),
        variant=ActionVariant.INFO,
        icon="sort",
    )
    def update_sort_order(self, request, queryset):
        countries = list(queryset.order_by("alpha_2"))
        for index, country in enumerate(countries):
            country.sort_order = index
            country.save(update_fields=["sort_order"])

        count = len(countries)
        self.message_user(
            request,
            _("Updated sort order for %(count)d countries.") % {"count": count},
            messages.SUCCESS,
        )
